@@ -6,12 +6,12 @@ TEST(coroutine, create)
 {
     long cid = Coroutine::create([](void *arg)
     {
-        long cid = coroutine_get_current_cid();
-        Coroutine *co = coroutine_get_by_id(cid);
+        long cid = Coroutine::get_current_cid();
+        Coroutine *co = Coroutine::get_by_cid(cid);
         co->yield();
     });
     ASSERT_GT(cid, 0);
-    coroutine_get_by_id(cid)->resume();
+    Coroutine::get_by_cid(cid)->resume();
 }
 
 TEST(coroutine, socket_connect_refused)
@@ -48,6 +48,58 @@ TEST(coroutine, socket_connect_with_dns)
     });
 }
 
+TEST(coroutine, socket_resolve_with_cache)
+{
+    coro_test([](void *arg)
+    {
+        set_dns_cache_capacity(10);
+
+        std::string addr1 = Coroutine::gethostbyname("www.baidu.com", AF_INET);
+        std::string addr2 = Coroutine::gethostbyname("www.baidu.com", AF_INET);
+
+        ASSERT_NE(addr1, "");
+        ASSERT_NE(addr2, "");
+        ASSERT_EQ(addr1, addr2);
+    });
+}
+
+TEST(coroutine, socket_resolve_without_cache)
+{
+    coro_test([](void *arg)
+    {
+        set_dns_cache_capacity(0);
+
+        std::string addr1 = Coroutine::gethostbyname("www.baidu.com", AF_INET);
+        std::string addr2 = Coroutine::gethostbyname("www.baidu.com", AF_INET);
+
+        ASSERT_NE(addr1, "");
+        ASSERT_NE(addr2, "");
+        ASSERT_NE(addr1, addr2);
+    });
+}
+
+TEST(coroutine, socket_resolve_cache_inet4_and_inet6)
+{
+    coro_test([](void *arg)
+    {
+        set_dns_cache_capacity(10);
+
+        std::string addr1 = Coroutine::gethostbyname("ipv6.sjtu.edu.cn", AF_INET);
+        std::string addr2 = Coroutine::gethostbyname("ipv6.sjtu.edu.cn", AF_INET6);
+
+        ASSERT_NE(addr1, "");
+        ASSERT_NE(addr2, "");
+        ASSERT_EQ(addr1.find(":"), addr1.npos);
+        ASSERT_NE(addr2.find(":"), addr2.npos);
+
+        std::string addr3 = Coroutine::gethostbyname("ipv6.sjtu.edu.cn", AF_INET);
+        std::string addr4 = Coroutine::gethostbyname("ipv6.sjtu.edu.cn", AF_INET6);
+
+        ASSERT_EQ(addr1, addr3);
+        ASSERT_EQ(addr2, addr4);
+    });
+}
+
 TEST(coroutine, socket_recv_success)
 {
     coro_test([](void *arg)
@@ -131,14 +183,3 @@ TEST(coroutine, socket_accept)
 
     coro_test(fns, 2);
 }
-
-TEST(coroutine, socket_resolve)
-{
-    coro_test([](void *arg)
-    {
-        Socket sock(SW_SOCK_TCP);
-        auto retval = sock.resolve("www.swoole.com");
-        ASSERT_EQ(retval, "47.244.108.17");
-    });
-}
-
