@@ -176,12 +176,7 @@ static sw_inline ssize_t swConnection_recv(swConnection *conn, void *__buf, size
     {
         goto _recv;
     }
-    else
-    {
-        goto _return;
-    }
 
-    _return:
 #ifdef SW_DEBUG
     if (total_bytes > 0)
     {
@@ -189,7 +184,7 @@ static sw_inline ssize_t swConnection_recv(swConnection *conn, void *__buf, size
     }
 #endif
 
-    swDebug("recv %ld/%ld bytes, errno=%d", total_bytes, __n, errno);
+    swTraceLog(SW_TRACE_SOCKET, "recv %ld/%ld bytes, errno=%d", total_bytes, __n, errno);
 
     return total_bytes;
 }
@@ -241,6 +236,7 @@ static sw_inline ssize_t swConnection_send(swConnection *conn, void *__buf, size
 static sw_inline ssize_t swConnection_peek(swConnection *conn, void *__buf, size_t __n, int __flags)
 {
     int retval;
+    __flags |= MSG_PEEK;
     _peek:
 #ifdef SW_USE_OPENSSL
     if (conn->ssl)
@@ -248,12 +244,10 @@ static sw_inline ssize_t swConnection_peek(swConnection *conn, void *__buf, size
         retval = SSL_peek(conn->ssl, __buf, __n);
     }
     else
+#endif
     {
         retval = recv(conn->fd, __buf, __n, __flags);
     }
-#else
-    retval = recv(conn->fd, __buf, __n, __flags);
-#endif
 
     if (retval < 0 && errno == EINTR)
     {
