@@ -34,6 +34,7 @@ enum swErrorCode
     SW_ERROR_FILE_EMPTY,
     SW_ERROR_DNSLOOKUP_DUPLICATE_REQUEST,
     SW_ERROR_DNSLOOKUP_RESOLVE_FAILED,
+    SW_ERROR_DNSLOOKUP_RESOLVE_TIMEOUT,
     SW_ERROR_BAD_IPV6_ADDRESS,
     SW_ERROR_UNREGISTERED_SIGNAL,
 
@@ -74,7 +75,8 @@ enum swErrorCode
     /**
      * AIO
      */
-    SW_ERROR_AIO_BAD_REQUEST   = 4001,
+    SW_ERROR_AIO_BAD_REQUEST = 4001,
+    SW_ERROR_AIO_CANCELED,
 
     /**
      * Client & Socket
