@@ -457,9 +457,9 @@ struct _swServer
      */
     uint32_t reload_async :1;
     /**
-     * asynchronous task worker
+     * enable coroutine in task worker
      */
-    uint32_t task_async :1;
+    uint32_t task_enable_coroutine :1;
     /**
      * slowlog
      */
@@ -468,6 +468,10 @@ struct _swServer
      * yield coroutine when the output buffer is full
      */
     uint32_t send_yield :1;
+    /**
+     * enable coroutine
+     */
+    uint32_t enable_coroutine :1;
 
     /**
      *  heartbeat check time
