@@ -24,55 +24,50 @@
 
 namespace swoole
 {
-
 class Socket
 {
 public:
-    swReactor *reactor;
-    std::string _host;
-    std::string bind_address;
-    int bind_port;
-    int _port;
-    Coroutine* bind_co;
-    swTimer_node *_timer;
+    enum timer_levels
+    {
+        TIMER_LV_NORMAL,
+        TIMER_LV_MULTI,
+        TIMER_LV_PACKET,
+        TIMER_LV_GLOBAL
+    };
+
     swConnection *socket = nullptr;
     enum swSocket_type type;
-    int _sock_type;
-    int _sock_domain;
-    double _timeout;
-    double _timeout_temp;
-    int _backlog;
-    bool _closed;
-    int errCode;
-    const char *errMsg;
-    uint32_t http2 :1;
-    uint32_t shutdown_read :1;
-    uint32_t shutdown_write :1;
-    /**
-     * one package: length check
-     */
-    uint32_t open_length_check :1;
-    uint32_t open_eof_check :1;
+    int sock_domain = 0;
+    int sock_type = 0;
+    int sock_protocol = 0;
+    int backlog = 0;
+    int errCode = 0;
+    const char *errMsg = "";
 
-    swProtocol protocol;
-    swString *read_buffer;
-    swString *write_buffer;
-    swSocketAddress bind_address_info;
+    bool open_length_check = false;
+    bool open_eof_check = false;
+    bool http2 = false;
 
-    struct _swSocks5 *socks5_proxy;
-    struct _http_proxy* http_proxy;
+    swProtocol protocol = {0};
+    swString *read_buffer = nullptr;
+    swString *write_buffer = nullptr;
+    swSocketAddress bind_address_info = {{}, 0};
+
+    struct _swSocks5 *socks5_proxy = nullptr;
+    struct _http_proxy* http_proxy = nullptr;
 
 #ifdef SW_USE_OPENSSL
-    bool open_ssl;
-    bool ssl_wait_handshake;
-    SSL_CTX *ssl_context;
-    swSSL_option ssl_option;
+    bool open_ssl = false;
+    swSSL_option ssl_option = {0};
 #endif
 
-    Socket(enum swSocket_type type);
+    Socket(int domain = AF_INET, int type = SOCK_STREAM, int protocol = 0);
+    Socket(enum swSocket_type type = SW_SOCK_TCP);
     Socket(int _fd, Socket *sock);
     Socket(int _fd, enum swSocket_type _type);
     ~Socket();
+    void set_timer(timer_levels _timer_level = TIMER_LV_NORMAL, double _timeout = 0);
+    void del_timer(timer_levels _timer_level = TIMER_LV_NORMAL);
     bool connect(std::string host, int port, int flags = 0);
     bool connect(const struct sockaddr *addr, socklen_t addrlen);
     bool shutdown(int how = SHUT_RDWR);
@@ -91,7 +86,6 @@ public:
     ssize_t recv_packet();
     Socket* accept();
     bool bind(std::string address, int port = 0);
-    std::string resolve(std::string host);
     bool listen(int backlog = 0);
     bool sendfile(char *filename, off_t offset, size_t length);
     ssize_t sendto(char *address, int port, char *data, int len);
@@ -103,63 +97,18 @@ public:
     bool ssl_accept();
 #endif
 
-    void yield();
-
-    inline void resume()
+    static inline enum swSocket_type get_type(int domain, int type, int protocol = 0)
     {
-        bind_co->resume();
-    }
-
-    inline bool wait_readable()
-    {
-#ifdef SW_USE_OPENSSL
-        if (socket->ssl && socket->ssl_want_write)
+        switch (domain)
         {
-            if (unlikely(!is_available() || !wait_event(SW_EVENT_WRITE)))
-            {
-                return false;
-            }
-        }
-        else
-#endif
-        if (unlikely(!wait_event(SW_EVENT_READ)))
-        {
-            return false;
-        }
-        yield();
-        return errCode != ETIMEDOUT;
-    }
-
-    inline bool wait_writeable(const void **__buf, size_t __n)
-    {
-#ifdef SW_USE_OPENSSL
-        if (socket->ssl && socket->ssl_want_read)
-        {
-            if (unlikely(!is_available() || !wait_event(SW_EVENT_READ)))
-            {
-                return false;
-            }
-        }
-        else
-#endif
-        if (unlikely(!wait_event(SW_EVENT_WRITE)))
-        {
-            return false;
-        }
-        copy_to_write_buffer(__buf, __n);
-        yield();
-        return errCode != ETIMEDOUT;
-    }
-
-    inline long has_bound()
-    {
-        if (bind_co)
-        {
-            return bind_co->get_cid();
-        }
-        else
-        {
-            return 0;
+        case AF_INET:
+            return type == SOCK_STREAM ? SW_SOCK_TCP : SW_SOCK_UDP;
+        case AF_INET6:
+            return type == SOCK_STREAM ? SW_SOCK_TCP6 : SW_SOCK_UDP6;
+        case AF_UNIX:
+            return type == SOCK_STREAM ? SW_SOCK_UNIX_STREAM : SW_SOCK_UNIX_DGRAM;
+        default:
+            return SW_SOCK_TCP;
         }
     }
 
@@ -168,6 +117,27 @@ public:
         return socket ? socket->fd : -1;
     }
 
+    inline long has_bound()
+    {
+        return coroutine ? coroutine->get_cid() : 0;
+    }
+
+    inline void check_bind()
+    {
+        long cid = has_bound();
+        if (unlikely(cid))
+        {
+            swoole_error_log(
+                SW_LOG_ERROR, SW_ERROR_CO_HAS_BEEN_BOUND,
+                "Socket#%d has already been bound to another coroutine#%ld, "
+                "reading or writing of the same socket in multiple coroutines at the same time is not allowed.\n",
+                socket->fd, cid
+            );
+            set_err(SW_ERROR_CO_HAS_BEEN_BOUND);
+            exit(255);
+        }
+    }
+
     inline void set_err(int e)
     {
         errCode = errno = e;
@@ -186,20 +156,18 @@ public:
         errMsg = swstrerror(e);
     }
 
-    inline void set_timeout(double timeout, bool temp = false)
+    inline double get_timeout()
+    {
+        return timeout;
+    }
+
+    inline void set_timeout(double timeout)
     {
         if (timeout == 0)
         {
             return;
         }
-        if (temp)
-        {
-            _timeout_temp = timeout;
-        }
-        else
-        {
-            _timeout = timeout;
-        }
+        this->timeout = timeout;
     }
 
     inline void set_timeout(struct timeval *timeout)
@@ -218,15 +186,12 @@ public:
             swSysError("setsockopt(%d, TCP_NODELAY) failed.", get_fd());
             return false;
         }
-        else
-        {
-            return true;
-        }
+        return true;
     }
 
     inline swString* get_read_buffer()
     {
-        if (unlikely(read_buffer == nullptr))
+        if (unlikely(!read_buffer))
         {
             read_buffer = swString_new(SW_BUFFER_SIZE_STD);
         }
@@ -235,13 +200,156 @@ public:
 
     inline swString* get_write_buffer()
     {
-        if (unlikely(write_buffer == nullptr))
+        if (unlikely(!write_buffer))
         {
             write_buffer = swString_new(SW_BUFFER_SIZE_STD);
         }
         return write_buffer;
     }
 
+protected:
+    Coroutine* coroutine = nullptr;
+    swReactor *reactor = nullptr;
+
+    std::string host;
+    int port = 0;
+    std::string bind_address;
+    int bind_port = 0;
+    timer_levels timer_level = TIMER_LV_NORMAL;
+    swTimer_node *timer = nullptr;
+    double timeout = -1;
+
+    bool shutdown_read = false;
+    bool shutdown_write = false;
+#ifdef SW_USE_OPENSSL
+    SSL_CTX *ssl_context = nullptr;
+#endif
+
+    static void timer_callback(swTimer *timer, swTimer_node *tnode);
+    static int event_callback(swReactor *reactor, swEvent *event);
+
+    bool socks5_handshake();
+    bool http_proxy_handshake();
+
+    inline void yield()
+    {
+        Coroutine *co = Coroutine::get_current();
+        if (unlikely(!co))
+        {
+            swError("Socket::yield() must be called in the coroutine.");
+        }
+        set_err(0);
+        set_timer();
+        coroutine = co;
+        co->yield();
+        coroutine = nullptr;
+        del_timer();
+    }
+
+    inline void resume()
+    {
+        coroutine->resume();
+    }
+
+    inline void init_members()
+    {
+        protocol.package_length_type = 'N';
+        protocol.package_length_size = 4;
+        protocol.package_body_offset = 0;
+        protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
+    }
+
+    inline void init_sock_type(enum swSocket_type _type)
+    {
+        type = _type;
+        switch (type)
+        {
+        case SW_SOCK_TCP6:
+            sock_domain = AF_INET6;
+            sock_type = SOCK_STREAM;
+            break;
+        case SW_SOCK_UNIX_STREAM:
+            sock_domain = AF_UNIX;
+            sock_type = SOCK_STREAM;
+            break;
+        case SW_SOCK_UDP:
+            sock_domain = AF_INET;
+            sock_type = SOCK_DGRAM;
+            break;
+        case SW_SOCK_UDP6:
+            sock_domain = AF_INET6;
+            sock_type = SOCK_DGRAM;
+            break;
+        case SW_SOCK_UNIX_DGRAM:
+            sock_domain = AF_UNIX;
+            sock_type = SOCK_DGRAM;
+            break;
+        case SW_SOCK_TCP:
+        default:
+            sock_domain = AF_INET;
+            sock_type = SOCK_STREAM;
+            break;
+        }
+    }
+
+    inline void init_sock();
+
+    inline void init_sock(int _fd);
+
+    inline bool is_available()
+    {
+        check_bind();
+        if (unlikely(socket->closed))
+        {
+            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SOCKET_CLOSED, "Socket#%d belongs to coroutine#%ld has already been closed.", socket->fd, Coroutine::get_current_cid());
+            set_err(ECONNRESET);
+            return false;
+        }
+        return true;
+    }
+
+    inline bool should_be_break()
+    {
+        switch (errCode)
+        {
+        case ETIMEDOUT:
+        case ECANCELED:
+            return true;
+        default:
+            return false;
+        }
+    }
+
+    inline bool wait_event(int event)
+    {
+        if (reactor->add(reactor, socket->fd, SW_FD_CORO_SOCKET | event) < 0)
+        {
+            set_err(errno);
+            return false;
+        }
+        return true;
+    }
+
+    inline bool wait_readable()
+    {
+#ifdef SW_USE_OPENSSL
+        if (socket->ssl && socket->ssl_want_write)
+        {
+            if (unlikely(!is_available() || !wait_event(SW_EVENT_WRITE)))
+            {
+                return false;
+            }
+        }
+        else
+#endif
+        if (unlikely(!wait_event(SW_EVENT_READ)))
+        {
+            return false;
+        }
+        yield();
+        return !should_be_break();
+    }
+
     inline void copy_to_write_buffer(const void **__buf, size_t __n)
     {
         if (*__buf != get_write_buffer()->str)
@@ -252,156 +360,28 @@ public:
         }
     }
 
-protected:
-    bool socks5_handshake();
-    bool http_proxy_handshake();
-
-    inline void init_members()
+    inline bool wait_writeable(const void **__buf = nullptr, size_t __n = 0)
     {
-        bind_co = nullptr;
-        _timeout = 0;
-        _timeout_temp = 0;
-        _port = 0;
-        errCode = 0;
-        errMsg = nullptr;
-        _timer = nullptr;
-        bind_port = 0;
-        _backlog = 0;
-
-        http2 = 0;
-        shutdown_read = 0;
-        shutdown_write = 0;
-        open_length_check = 0;
-        open_eof_check = 0;
-
-        socks5_proxy = nullptr;
-        http_proxy = nullptr;
-
-        read_buffer = nullptr;
-        write_buffer = nullptr;
-        protocol = {0};
-        bind_address_info = {{}, 0};
-
-        protocol.package_length_type = 'N';
-        protocol.package_length_size = 4;
-        protocol.package_body_offset = 0;
-        protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
-
 #ifdef SW_USE_OPENSSL
-        open_ssl = false;
-        ssl_context = NULL;
-        ssl_option = {0};
+        if (socket->ssl && socket->ssl_want_read)
+        {
+            if (unlikely(!is_available() || !wait_event(SW_EVENT_READ)))
+            {
+                return false;
+            }
+        }
+        else
 #endif
-    }
-
-    inline void init_sock_type(enum swSocket_type _type)
-    {
-        type = _type;
-        switch (type)
+        if (unlikely(!wait_event(SW_EVENT_WRITE)))
         {
-        case SW_SOCK_TCP6:
-            _sock_domain = AF_INET6;
-            _sock_type = SOCK_STREAM;
-            break;
-        case SW_SOCK_UNIX_STREAM:
-            _sock_domain = AF_UNIX;
-            _sock_type = SOCK_STREAM;
-            break;
-        case SW_SOCK_UDP:
-            _sock_domain = AF_INET;
-            _sock_type = SOCK_DGRAM;
-            break;
-        case SW_SOCK_UDP6:
-            _sock_domain = AF_INET6;
-            _sock_type = SOCK_DGRAM;
-            break;
-        case SW_SOCK_UNIX_DGRAM:
-            _sock_domain = AF_UNIX;
-            _sock_type = SOCK_DGRAM;
-            break;
-        case SW_SOCK_TCP:
-        default:
-            _sock_domain = AF_INET;
-            _sock_type = SOCK_STREAM;
-            break;
-        }
-    }
-
-    inline void init_sock(int _fd);
-
-    inline bool wait_event(int event)
-    {
-        if (reactor->add(reactor, socket->fd, SW_FD_CORO_SOCKET | event) < 0)
-        {
-            set_err(errno);
             return false;
         }
-        return true;
-    }
-
-    inline bool is_available(bool allow_cross_co = false)
-    {
-        long cid = has_bound();
-        if (unlikely(!allow_cross_co && cid))
+        if (__n > 0)
         {
-            swoole_error_log(
-                SW_LOG_ERROR, SW_ERROR_CO_HAS_BEEN_BOUND,
-                "Socket#%d has already been bound to another coroutine#%ld, "
-                "reading or writing of the same socket in multiple coroutines at the same time is not allowed.\n",
-                socket->fd, cid
-            );
-            set_err(SW_ERROR_CO_HAS_BEEN_BOUND);
-            exit(255);
+            copy_to_write_buffer(__buf, __n);
         }
-        if (unlikely(socket->closed))
-        {
-            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SOCKET_CLOSED, "Socket#%d belongs to coroutine#%ld has already been closed.", socket->fd, cid);
-            set_err(ECONNRESET);
-            return false;
-        }
-        return true;
+        yield();
+        return !should_be_break();
     }
 };
-
-static inline enum swSocket_type get_socket_type(int domain, int type, int protocol)
-{
-    if (domain == AF_INET)
-    {
-        return type == SOCK_STREAM ? SW_SOCK_TCP : SW_SOCK_UDP;
-    }
-    else if (domain == AF_INET6)
-    {
-        return type == SOCK_STREAM ? SW_SOCK_TCP6 : SW_SOCK_UDP6;
-    }
-    else if (domain == AF_UNIX)
-    {
-        return type == SOCK_STREAM ? SW_SOCK_UNIX_STREAM : SW_SOCK_UNIX_DGRAM;
-    }
-    else
-    {
-        return SW_SOCK_TCP;
-    }
-}
-
-static inline enum swSocket_type get_socket_type_from_uri(std::string &uri, bool convert_to_addr = false)
-{
-    if (uri.compare(0, 6, "unix:/", 0, 6) == 0)
-    {
-        if (convert_to_addr)
-        {
-            uri = uri.substr(sizeof("unix:") - 1);
-            uri.erase(0, uri.find_first_not_of('/') - 1);
-        }
-        return SW_SOCK_UNIX_STREAM;
-    }
-    else if (uri.find(':') != std::string::npos)
-    {
-        return SW_SOCK_TCP6;
-    }
-    else
-    {
-        return SW_SOCK_TCP;
-    }
-}
-
 };
