@@ -19,9 +19,7 @@
 
 #if defined(HAVE_CONFIG_H) && !defined(COMPILE_DL_SWOOLE)
 #include "config.h"
-#endif
-
-#ifdef SW_STATIC_COMPILATION
+#elif defined(PHP_ATOM_INC) || defined(ZEND_SIGNALS)
 #include "php_config.h"
 #endif
 
@@ -89,10 +87,10 @@ int clock_gettime(clock_id_t which_clock, struct timespec *t);
 
 #define SWOOLE_MAJOR_VERSION      4
 #define SWOOLE_MINOR_VERSION      2
-#define SWOOLE_RELEASE_VERSION    10
+#define SWOOLE_RELEASE_VERSION    12
 #define SWOOLE_EXTRA_VERSION      ""
-#define SWOOLE_VERSION            "4.2.10"
-#define SWOOLE_VERSION_ID         40210
+#define SWOOLE_VERSION            "4.2.12"
+#define SWOOLE_VERSION_ID         40212
 #define SWOOLE_BUG_REPORT \
     "A bug occurred in Swoole-v" SWOOLE_VERSION ", please report it.\n"\
     "The Swoole developers probably don't know about it,\n"\
@@ -286,8 +284,9 @@ enum swFd_type
     SW_FD_INOTIFY         = 13, //server socket
     SW_FD_CHAN_PIPE       = 14, //channel pipe
     SW_FD_USER            = 15, //SW_FD_USER or SW_FD_USER+n: for custom event
-    SW_FD_STREAM_CLIENT   = 16, //swClient stream
-    SW_FD_DGRAM_CLIENT    = 17, //swClient dgram
+    SW_FD_ARES            = 16, //c-ares
+    SW_FD_STREAM_CLIENT   = 17, //swClient stream
+    SW_FD_DGRAM_CLIENT    = 18, //swClient dgram
 };
 
 enum swBool_type
@@ -2015,6 +2014,9 @@ int swProtocol_recv_check_length(swProtocol *protocol, swConnection *conn, swStr
 int swProtocol_recv_check_eof(swProtocol *protocol, swConnection *conn, swString *buffer);
 
 //--------------------------------timer------------------------------
+#define SW_TIMER_MAX_MS  LONG_MAX
+#define SW_TIMER_MAX_SEC (LONG_MAX / 1000)
+
 typedef struct _swTimer swTimer;
 typedef struct _swTimer_node swTimer_node;
 
