@@ -284,6 +284,7 @@ PHP_FUNCTION(swoole_set_process_name);
 PHP_FUNCTION(swoole_get_local_ip);
 PHP_FUNCTION(swoole_get_local_mac);
 PHP_FUNCTION(swoole_call_user_shutdown_begin);
+PHP_FUNCTION(swoole_clear_dns_cache);
 //---------------------------------------------------------
 //                  Coroutine API
 //---------------------------------------------------------
@@ -292,61 +293,7 @@ PHP_FUNCTION(swoole_coroutine_exec);
 PHP_FUNCTION(swoole_coroutine_gethostbyname);
 PHP_FUNCTION(swoole_coroutine_defer);
 //---------------------------------------------------------
-//                  swoole_server
-//---------------------------------------------------------
-PHP_METHOD(swoole_server, __construct);
-PHP_METHOD(swoole_server, __destruct);
-PHP_METHOD(swoole_server, set);
-PHP_METHOD(swoole_server, on);
-PHP_METHOD(swoole_server, listen);
-PHP_METHOD(swoole_server, sendMessage);
-PHP_METHOD(swoole_server, addProcess);
-PHP_METHOD(swoole_server, start);
-PHP_METHOD(swoole_server, stop);
-PHP_METHOD(swoole_server, send);
-PHP_METHOD(swoole_server, sendfile);
-PHP_METHOD(swoole_server, stats);
-PHP_METHOD(swoole_server, bind);
-PHP_METHOD(swoole_server, sendto);
-PHP_METHOD(swoole_server, sendwait);
-PHP_METHOD(swoole_server, exist);
-PHP_METHOD(swoole_server, protect);
-PHP_METHOD(swoole_server, close);
-PHP_METHOD(swoole_server, confirm);
-PHP_METHOD(swoole_server, pause);
-PHP_METHOD(swoole_server, resume);
-PHP_METHOD(swoole_server, task);
-PHP_METHOD(swoole_server, taskwait);
-PHP_METHOD(swoole_server, taskWaitMulti);
-PHP_METHOD(swoole_server, taskCo);
-PHP_METHOD(swoole_server, finish);
-PHP_METHOD(swoole_server, reload);
-PHP_METHOD(swoole_server, shutdown);
-PHP_METHOD(swoole_server, getLastError);
-PHP_METHOD(swoole_server, heartbeat);
-PHP_METHOD(swoole_server, connection_list);
-PHP_METHOD(swoole_server, connection_info);
-#ifdef SW_BUFFER_RECV_TIME
-PHP_METHOD(swoole_server, getReceivedTime);
-#endif
-
-PHP_METHOD(swoole_connection_iterator, count);
-PHP_METHOD(swoole_connection_iterator, rewind);
-PHP_METHOD(swoole_connection_iterator, next);
-PHP_METHOD(swoole_connection_iterator, current);
-PHP_METHOD(swoole_connection_iterator, key);
-PHP_METHOD(swoole_connection_iterator, valid);
-PHP_METHOD(swoole_connection_iterator, offsetExists);
-PHP_METHOD(swoole_connection_iterator, offsetGet);
-PHP_METHOD(swoole_connection_iterator, offsetSet);
-PHP_METHOD(swoole_connection_iterator, offsetUnset);
-PHP_METHOD(swoole_connection_iterator, __destruct);
-
-#ifdef SWOOLE_SOCKETS_SUPPORT
-PHP_METHOD(swoole_server, getSocket);
-#endif
-//---------------------------------------------------------
-//                  swoole_event
+//                  event
 //---------------------------------------------------------
 PHP_FUNCTION(swoole_event_add);
 PHP_FUNCTION(swoole_event_set);
@@ -360,7 +307,7 @@ PHP_FUNCTION(swoole_event_dispatch);
 PHP_FUNCTION(swoole_event_isset);
 PHP_FUNCTION(swoole_client_select);
 //---------------------------------------------------------
-//                  swoole_async
+//                  async
 //---------------------------------------------------------
 PHP_FUNCTION(swoole_async_read);
 PHP_FUNCTION(swoole_async_write);
@@ -372,17 +319,18 @@ PHP_FUNCTION(swoole_async_dns_lookup_coro);
 PHP_FUNCTION(swoole_async_set);
 PHP_METHOD(swoole_async, exec);
 //---------------------------------------------------------
-//                  swoole_timer
+//                  timer
 //---------------------------------------------------------
 PHP_FUNCTION(swoole_timer_after);
 PHP_FUNCTION(swoole_timer_tick);
 PHP_FUNCTION(swoole_timer_exists);
 PHP_FUNCTION(swoole_timer_clear);
 //---------------------------------------------------------
-//                  other
+//                  error
 //---------------------------------------------------------
 PHP_FUNCTION(swoole_strerror);
 PHP_FUNCTION(swoole_errno);
+PHP_FUNCTION(swoole_last_error);
 //---------------------------------------------------------
 //                  serialize
 //---------------------------------------------------------
@@ -394,6 +342,7 @@ PHP_FUNCTION(swoole_unserialize);
 
 void swoole_destroy_table(zend_resource *rsrc);
 
+void swoole_server_init(int module_number);
 void swoole_server_port_init(int module_number);
 void swoole_async_init(int module_number);
 void swoole_table_init(int module_number);
@@ -446,7 +395,6 @@ static sw_inline void php_swoole_check_reactor()
     }
 }
 
-void php_swoole_check_aio();
 void php_swoole_register_shutdown_function(char *function);
 void php_swoole_register_shutdown_function_prepend(char *function);
 void php_swoole_event_init();
@@ -546,7 +494,6 @@ PHPAPI int php_swoole_unserialize(void *buffer, size_t len, zval *return_value,
 
 #ifdef SW_COROUTINE
 int php_coroutine_reactor_can_exit(swReactor *reactor);
-void sw_coro_check_bind(const char *name, long bind_cid);
 #endif
 
 #ifdef SW_USE_OPENSSL
@@ -556,7 +503,6 @@ void php_swoole_client_check_ssl_setting(swClient *cli, zval *zset);
 void php_swoole_class_unset_property_deny(zval *zobject, zval *member, void **cache_slot);
 
 ZEND_BEGIN_MODULE_GLOBALS(swoole)
-    long aio_thread_num;
     zend_bool display_errors;
     zend_bool cli;
     zend_bool use_namespace;
@@ -642,6 +588,9 @@ static sw_inline void _sw_zend_bailout(const char *filename, uint32_t lineno)
 
 /* PHP 7.0 compatibility macro {{{*/
 #if PHP_VERSION_ID < 70100
+// Fixed typo error in (https://github.com/php/php-src/commit/4c9e4caab40c5a1b3c8a52ad06c21175d091c3e4)
+#define ZEND_VM_STACK_ELEMENTS ZEND_VM_STACK_ELEMETS
+// Fixed >= 7.2 by using (EG(fake_scope))
 #define SW_DECLARE_EG_SCOPE(_scope) zend_class_entry *_scope
 #define SW_SAVE_EG_SCOPE(_scope) _scope = EG(scope)
 #define SW_SET_EG_SCOPE(_scope) EG(scope) = _scope
@@ -651,34 +600,34 @@ static sw_inline void _sw_zend_bailout(const char *filename, uint32_t lineno)
 #define SW_SET_EG_SCOPE(scope)
 #endif/*}}}*/
 
-/* PHP 7.3 forward compatibility */
+/* PHP 7.3 compatibility macro {{{*/
 #ifndef GC_SET_REFCOUNT
 # define GC_SET_REFCOUNT(p, rc) do { \
 		GC_REFCOUNT(p) = rc; \
 	} while (0)
 #endif
 
+#ifndef GC_ADDREF
+#define GC_ADDREF(ref) ++GC_REFCOUNT(ref)
+#define GC_DELREF(ref) --GC_REFCOUNT(ref)
+#endif
+
 #ifndef GC_IS_RECURSIVE
-# define GC_IS_RECURSIVE(p) \
+#define GC_IS_RECURSIVE(p) \
 	(ZEND_HASH_GET_APPLY_COUNT(p) > 1)
-# define GC_PROTECT_RECURSION(p) \
+#define GC_PROTECT_RECURSION(p) \
 	ZEND_HASH_INC_APPLY_COUNT(p)
-# define GC_UNPROTECT_RECURSION(p) \
+#define GC_UNPROTECT_RECURSION(p) \
 	ZEND_HASH_DEC_APPLY_COUNT(p)
 #endif
 
-/* PHP 7.3 compatibility macro {{{*/
 #ifndef ZEND_CLOSURE_OBJECT
-# define ZEND_CLOSURE_OBJECT(func) (zend_object*)func->op_array.prototype
+#define ZEND_CLOSURE_OBJECT(func) (zend_object*)func->op_array.prototype
 #endif
-#ifndef GC_ADDREF
-# define GC_ADDREF(ref) ++GC_REFCOUNT(ref)
-# define GC_DELREF(ref) --GC_REFCOUNT(ref)
-#endif/*}}}*/
 
 #ifndef ZEND_HASH_APPLY_PROTECTION
-# define ZEND_HASH_APPLY_PROTECTION(p) 1
-#endif
+#define ZEND_HASH_APPLY_PROTECTION(p) 1
+#endif/*}}}*/
 
 /* PHP 7 wrapper functions / macros */
 
@@ -996,16 +945,13 @@ static sw_inline void sw_fci_cache_persist(zend_fcall_info_cache *fci_cache)
 
 static sw_inline void sw_fci_cache_discard(zend_fcall_info_cache *fci_cache)
 {
-    zval _zobject, *zobject = &_zobject;
     if (fci_cache->object)
     {
-        ZVAL_OBJ(zobject, fci_cache->object);
-        zval_ptr_dtor(zobject);
+        OBJ_RELEASE(fci_cache->object);
     }
     if (fci_cache->function_handler->op_array.fn_flags & ZEND_ACC_CLOSURE)
     {
-        ZVAL_OBJ(zobject, ZEND_CLOSURE_OBJECT(fci_cache->function_handler));
-        zval_ptr_dtor(zobject);
+        OBJ_RELEASE(ZEND_CLOSURE_OBJECT(fci_cache->function_handler));
     }
 }
 
