@@ -38,8 +38,10 @@ swString *swString_new(size_t size)
 
 void swString_print(swString *str)
 {
-    printf("String[length=%d,size=%d,offset=%d]=%s\n", (int) str->length, (int) str->size, (int) str->offset,
-            str->str);
+    printf(
+        "String[length=%d,size=%d,offset=%d]=%s\n",
+        (int) str->length, (int) str->size, (int) str->offset, str->str
+    );
 }
 
 swString *swString_dup2(swString *src)
@@ -106,13 +108,12 @@ int swString_append_int(swString *str, int value)
             return SW_ERR;
         }
     }
-
     memcpy(str->str + str->length, buf, s_len);
     str->length += s_len;
     return SW_OK;
 }
 
-int swString_append_ptr(swString *str, const char *append_str, int length)
+int swString_append_ptr(swString *str, const char *append_str, size_t length)
 {
     int new_size = str->length + length;
     if (new_size > str->size)
@@ -137,18 +138,15 @@ int swString_write(swString *str, off_t offset, swString *write_str)
             return SW_ERR;
         }
     }
-
     memcpy(str->str + offset, write_str->str, write_str->length);
-
     if (new_length > str->length)
     {
         str->length = new_length;
     }
-
     return SW_OK;
 }
 
-int swString_write_ptr(swString *str, off_t offset, char *write_str, int length)
+int swString_write_ptr(swString *str, off_t offset, char *write_str, size_t length)
 {
     int new_length = offset + length;
     if (new_length > str->size)
@@ -158,14 +156,11 @@ int swString_write_ptr(swString *str, off_t offset, char *write_str, int length)
             return SW_ERR;
         }
     }
-
     memcpy(str->str + offset, write_str, length);
-
     if (new_length > str->length)
     {
         str->length = new_length;
     }
-
     return SW_OK;
 }
 
@@ -278,13 +273,13 @@ size_t swoole_utf8_length(u_char *p, size_t n)
     return len;
 }
 
-static char characters[] =
-{ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
-        'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
-        't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', };
-
 void swoole_random_string(char *buf, size_t size)
 {
+    static char characters[] = {
+        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
+        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
+        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
+    };
     int i;
     for (i = 0; i < size; i++)
     {
