@@ -17,6 +17,7 @@
 #include "socket.h"
 #include "async.h"
 #include "coroutine.h"
+#include "lru_cache.h"
 
 #ifndef _WIN32
 
@@ -32,6 +33,27 @@
 using namespace swoole;
 using namespace std;
 
+static size_t dns_cache_capacity = 1000;
+static time_t dns_cache_expire = 60;
+static LRUCache *dns_cache = nullptr;
+
+void swoole::set_dns_cache_expire(time_t expire)
+{
+    dns_cache_expire = expire;
+}
+
+void swoole::set_dns_cache_capacity(size_t capacity)
+{
+    dns_cache_capacity = capacity;
+    delete dns_cache;
+    dns_cache = nullptr;
+}
+
+void swoole::clear_dns_cache()
+{
+    dns_cache->clear();
+}
+
 extern "C"
 {
 struct aio_task
@@ -42,12 +64,11 @@ struct aio_task
 
 int swoole_coroutine_socket(int domain, int type, int protocol)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return socket(domain, type, protocol);
     }
-    enum swSocket_type sock_type = get_socket_type(domain, type, protocol);
-    Socket *sock = new Socket(sock_type);
+    Socket *sock = new Socket(domain, type, protocol);
     if (sock->socket == nullptr)
     {
         delete sock;
@@ -58,7 +79,7 @@ int swoole_coroutine_socket(int domain, int type, int protocol)
 
 ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::send(sockfd, buf, len, flags);
     }
@@ -81,7 +102,7 @@ ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags
 
 ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::sendmsg(sockfd, msg, flags);
     }
@@ -104,7 +125,7 @@ ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags
 
 ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::recvmsg(sockfd, msg, flags);
     }
@@ -127,7 +148,7 @@ ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags)
 
 ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return ::recv(sockfd, buf, len, flags);
     }
@@ -158,7 +179,7 @@ ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags)
 
 int swoole_coroutine_close(int fd)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return close(fd);
     }
@@ -176,7 +197,7 @@ int swoole_coroutine_close(int fd)
 
 int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         _no_coro: return connect(sockfd, addr, addrlen);
     }
@@ -198,7 +219,7 @@ int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t
 
 int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current() || nfds != 1))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current() || nfds != 1))
     {
         _poll: return poll(fds, nfds, timeout);
     }
@@ -317,9 +338,31 @@ static void aio_onWriteFileCompleted(swAio_event *event)
     ((Coroutine *) task->co)->resume();
 }
 
+static void aio_onDNSCompleted(swAio_event *event)
+{
+    if (event->canceled)
+    {
+        return;
+    }
+    aio_task *task = (aio_task *) event->object;
+    task->event->ret = event->ret;
+    task->event->error = event->error;
+    ((Coroutine *) task->co)->resume();
+}
+
+static void aio_onDNSTimeout(swTimer *timer, swTimer_node *tnode)
+{
+    swAio_event *event = (swAio_event *) tnode->data;
+    event->canceled = 1;
+    aio_task *task = (aio_task *) event->object;
+    task->event->ret = -1;
+    task->event->error = SW_ERROR_DNSLOOKUP_RESOLVE_TIMEOUT;
+    ((Coroutine *) task->co)->resume();
+}
+
 int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return open(pathname, flags, mode);
     }
@@ -331,7 +374,7 @@ int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
     ev.flags = flags;
     ev.handler = handler_open;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -345,7 +388,7 @@ int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
 
 ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return read(fd, buf, count);
     }
@@ -364,7 +407,7 @@ ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
     ev.nbytes = count;
     ev.handler = handler_read;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -378,7 +421,7 @@ ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
 
 ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return write(fd, buf, count);
     }
@@ -397,7 +440,7 @@ ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
     ev.nbytes = count;
     ev.handler = handler_write;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -411,7 +454,7 @@ ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
 
 off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return lseek(fd, offset, whence);
     }
@@ -423,7 +466,7 @@ off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
     ev.flags = whence;
     ev.handler = handler_lseek;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -437,7 +480,7 @@ off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
 
 int swoole_coroutine_fstat(int fd, struct stat *statbuf)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return fstat(fd, statbuf);
     }
@@ -448,7 +491,7 @@ int swoole_coroutine_fstat(int fd, struct stat *statbuf)
     ev.buf = (void*) statbuf;
     ev.handler = handler_fstat;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -462,7 +505,7 @@ int swoole_coroutine_fstat(int fd, struct stat *statbuf)
 
 int swoole_coroutine_unlink(const char *pathname)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return unlink(pathname);
     }
@@ -472,7 +515,7 @@ int swoole_coroutine_unlink(const char *pathname)
     ev.buf = (void*) pathname;
     ev.handler = handler_unlink;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -486,7 +529,7 @@ int swoole_coroutine_unlink(const char *pathname)
 
 int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return statvfs(path, buf);
     }
@@ -497,7 +540,7 @@ int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
     ev.offset = (off_t) buf;
     ev.handler = handler_statvfs;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -511,7 +554,7 @@ int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
 
 int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return mkdir(pathname, mode);
     }
@@ -522,7 +565,7 @@ int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
     ev.offset = mode;
     ev.handler = handler_mkdir;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -536,7 +579,7 @@ int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
 
 int swoole_coroutine_rmdir(const char *pathname)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return rmdir(pathname);
     }
@@ -546,7 +589,7 @@ int swoole_coroutine_rmdir(const char *pathname)
     ev.buf = (void*) pathname;
     ev.handler = handler_rmdir;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -560,7 +603,7 @@ int swoole_coroutine_rmdir(const char *pathname)
 
 int swoole_coroutine_rename(const char *oldpath, const char *newpath)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return rename(oldpath, newpath);
     }
@@ -571,7 +614,7 @@ int swoole_coroutine_rename(const char *oldpath, const char *newpath)
     ev.offset = (off_t) newpath;
     ev.handler = handler_rename;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -585,7 +628,7 @@ int swoole_coroutine_rename(const char *oldpath, const char *newpath)
 
 int swoole_coroutine_access(const char *pathname, int mode)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return access(pathname, mode);
     }
@@ -596,7 +639,7 @@ int swoole_coroutine_access(const char *pathname, int mode)
     ev.offset = mode;
     ev.handler = handler_access;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -610,7 +653,7 @@ int swoole_coroutine_access(const char *pathname, int mode)
 
 int swoole_coroutine_flock(int fd, int operation)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return flock(fd, operation);
     }
@@ -621,7 +664,7 @@ int swoole_coroutine_flock(int fd, int operation)
     ev.flags = operation;
     ev.handler = handler_flock;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -640,8 +683,8 @@ static void sleep_timeout(swTimer *timer, swTimer_node *tnode)
 
 int Coroutine::sleep(double sec)
 {
-    Coroutine* co = coroutine_get_current();
-    if (swTimer_add(&SwooleG.timer, sec * 1000, 0, co, sleep_timeout) == NULL)
+    Coroutine* co = Coroutine::get_current();
+    if (swTimer_add(&SwooleG.timer, (long) (sec * 1000), 0, co, sleep_timeout) == NULL)
     {
         return -1;
     }
@@ -656,7 +699,7 @@ swString* Coroutine::read_file(const char *file, int lock)
     swAio_event ev;
     bzero(&ev, sizeof(swAio_event));
 
-    task.co = coroutine_get_current();
+    task.co = Coroutine::get_current();
     task.event = &ev;
 
     ev.lock = lock ? 1 : 0;
@@ -693,7 +736,7 @@ ssize_t Coroutine::write_file(const char *file, char *buf, size_t length, int lo
     swAio_event ev;
     bzero(&ev, sizeof(swAio_event));
 
-    task.co = coroutine_get_current();
+    task.co = Coroutine::get_current();
     task.event = &ev;
 
     ev.lock = lock ? 1 : 0;
@@ -719,6 +762,88 @@ ssize_t Coroutine::write_file(const char *file, char *buf, size_t length, int lo
     return ev.ret;
 }
 
+string Coroutine::gethostbyname(const string &hostname, int domain, double timeout)
+{
+    if (dns_cache == nullptr && dns_cache_capacity != 0)
+    {
+        dns_cache = new LRUCache(dns_cache_capacity);
+    }
+
+    string cache_key;
+    if (dns_cache)
+    {
+        cache_key.append(domain == AF_INET ? "4_" : "6_");
+        cache_key.append(hostname);
+        auto cache = dns_cache->get(cache_key);
+
+        if (cache)
+        {
+            return *(string *)cache.get();
+        }
+    }
+
+    swAio_event ev;
+    aio_task task ;
+
+    bzero(&ev, sizeof(swAio_event));
+    if (hostname.size() < SW_IP_MAX_LENGTH)
+    {
+        ev.nbytes = SW_IP_MAX_LENGTH + 1;
+    }
+    else
+    {
+        ev.nbytes = hostname.size() + 1;
+    }
+    ev.buf = sw_malloc(ev.nbytes);
+    if (!ev.buf)
+    {
+        return "";
+    }
+
+    task.co = Coroutine::get_current();
+    task.event = &ev;
+
+    memcpy(ev.buf, hostname.c_str(), hostname.size());
+    ((char *) ev.buf)[hostname.size()] = 0;
+    ev.flags = domain;
+    ev.type = SW_AIO_GETHOSTBYNAME;
+    ev.object = (void*) &task;
+    ev.handler = swAio_handler_gethostbyname;
+    ev.callback = aio_onDNSCompleted;
+
+    swAio_event *event = swAio_dispatch2(&ev);
+    swTimer_node* timer = nullptr;
+    if (timeout > 0)
+    {
+        timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, event, aio_onDNSTimeout);
+    }
+    task.co->yield();
+    if (timer)
+    {
+        swTimer_del(&SwooleG.timer, timer);
+    }
+
+    if (ev.ret == -1)
+    {
+        SwooleG.error = ev.error;
+        return "";
+    }
+    else
+    {
+        if (dns_cache)
+        {
+            string *addr = new string((char *) ev.buf);
+            dns_cache->set(cache_key, shared_ptr<string>(addr), dns_cache_expire);
+            sw_free(ev.buf);
+            return *addr;
+        }
+
+        string addr((char *) ev.buf);
+        sw_free(ev.buf);
+        return addr;
+    }
+}
+
 #if 0
 static void handler_opendir(swAio_event *event)
 {
@@ -736,7 +861,7 @@ static void handler_readdir(swAio_event *event)
 
 DIR *swoole_coroutine_opendir(const char *name)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return opendir(name);
     }
@@ -746,7 +871,7 @@ DIR *swoole_coroutine_opendir(const char *name)
     ev.buf = (void*) name;
     ev.handler = handler_opendir;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
@@ -760,7 +885,7 @@ DIR *swoole_coroutine_opendir(const char *name)
 
 struct dirent *swoole_coroutine_readdir(DIR *dirp)
 {
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return readdir(dirp);
     }
@@ -770,7 +895,7 @@ struct dirent *swoole_coroutine_readdir(DIR *dirp)
     ev.buf = (void*) dirp;
     ev.handler = handler_readdir;
     ev.callback = aio_onCompleted;
-    ev.object = coroutine_get_current();
+    ev.object = Coroutine::get_current();
     ev.req = &ev;
 
     int ret = swAio_dispatch(&ev);
