@@ -2,6 +2,7 @@
 #include "coroutine.h"
 #include "async.h"
 #include "buffer.h"
+#include "base64.h"
 
 #include <string>
 #include <iostream>
@@ -10,9 +11,23 @@
 using namespace swoole;
 using namespace std;
 
-static int socket_event_callback(swReactor *reactor, swEvent *event);
-static void socket_timer_callback(swTimer *timer, swTimer_node *tnode);
-static void socket_dns_callback(swAio_event *event);
+void Socket::timer_callback(swTimer *timer, swTimer_node *tnode)
+{
+    Socket *sock = (Socket *) tnode->data;
+    swTraceLog(SW_TRACE_SOCKET, "socket[%d] timeout", sock->socket->fd);
+    sock->set_err(ETIMEDOUT);
+    sock->reactor->del(sock->reactor, sock->socket->fd);
+    sock->timer = NULL;
+    sock->resume();
+}
+
+int Socket::event_callback(swReactor *reactor, swEvent *event)
+{
+    Socket *sock = (Socket *) event->socket->object;
+    sock->reactor->del(sock->reactor, event->fd);
+    sock->resume();
+    return SW_OK;
+}
 
 bool Socket::socks5_handshake()
 {
@@ -167,24 +182,29 @@ bool Socket::http_proxy_handshake()
     int n;
     if (http_proxy->password)
     {
-        if (strlen(http_proxy->buf) > 0) 
-        {
-            char _authBuf[700];
-            snprintf(_authBuf, sizeof(_authBuf), "CONNECT %*s:%d HTTP/1.1\r\n%s\r\n\r\n",
-                http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port, http_proxy->buf);
-            n = snprintf(http_proxy->buf, sizeof(http_proxy->buf), "%s", _authBuf);
-        }
-        else
-        {
-            n = snprintf(http_proxy->buf, sizeof(http_proxy->buf), "CONNECT %*s:%d HTTP/1.1\r\n\r\n",
-                    http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port);
-        }
+        char auth_buf[256];
+        char encode_buf[512];
+        n = snprintf(
+            auth_buf, sizeof(auth_buf), "%*s:%*s",
+            http_proxy->l_user, http_proxy->user,
+            http_proxy->l_password, http_proxy->password
+        );
+        swBase64_encode((unsigned char *) auth_buf, n, encode_buf);
+        n = snprintf(
+            http_proxy->buf, sizeof(http_proxy->buf),
+            "CONNECT %*s:%d HTTP/1.1\r\nProxy-Authorization:Basic %s\r\n\r\n",
+            http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port, encode_buf
+        );
     }
     else
     {
-        n = snprintf(http_proxy->buf, sizeof(http_proxy->buf), "CONNECT %*s:%d HTTP/1.1\r\n\r\n",
-                    http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port);
+        n = snprintf(
+            http_proxy->buf, sizeof(http_proxy->buf),
+            "CONNECT %*s:%d HTTP/1.1\r\n\r\n",
+            http_proxy->l_target_host, http_proxy->target_host, http_proxy->target_port
+        );
     }
+
     if (send(http_proxy->buf, n) <= 0)
     {
         return false;
@@ -272,6 +292,20 @@ static inline int socket_connect(int fd, const struct sockaddr *addr, socklen_t
     return retval;
 }
 
+void Socket::init_sock()
+{
+#ifdef SOCK_CLOEXEC
+    int _fd = ::socket(sock_domain, sock_type | SOCK_CLOEXEC, sock_protocol);
+#else
+    int _fd = ::socket(sock_domain, sock_type, sock_protocol);
+#endif
+    if (unlikely(_fd < 0))
+    {
+        swWarn("Socket construct failed. Error: %s[%d]", strerror(errno), errno);
+        return;
+    }
+    init_sock(_fd);
+}
 
 void Socket::init_sock(int _fd)
 {
@@ -295,27 +329,25 @@ void Socket::init_sock(int _fd)
     swSetNonBlock(socket->fd);
     if (!swReactor_handle_isset(reactor, SW_FD_CORO_SOCKET))
     {
-        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_READ, socket_event_callback);
-        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_WRITE, socket_event_callback);
-        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_ERROR, socket_event_callback);
+        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_READ, event_callback);
+        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_WRITE, event_callback);
+        reactor->setHandle(reactor, SW_FD_CORO_SOCKET | SW_EVENT_ERROR, event_callback);
     }
 }
 
+Socket::Socket(int _domain, int _type, int _protocol) :
+        sock_domain(_domain), sock_type(_type), sock_protocol(_protocol)
+{
+    init_members();
+    type = get_type(_domain, _type, _protocol);
+    init_sock();
+}
+
 Socket::Socket(enum swSocket_type _type)
 {
     init_members();
     init_sock_type(_type);
-#ifdef SOCK_CLOEXEC
-    int _fd = ::socket(_sock_domain, _sock_type | SOCK_CLOEXEC, 0);
-#else
-    int _fd = ::socket(_sock_domain, _sock_type, 0);
-#endif
-    if (unlikely(_fd < 0))
-    {
-        swWarn("Socket construct failed. Error: %s[%d]", strerror(errno), errno);
-        return;
-    }
-    init_sock(_fd);
+    init_sock();
 }
 
 Socket::Socket(int _fd, enum swSocket_type _type)
@@ -330,8 +362,8 @@ Socket::Socket(int _fd, Socket *server_sock)
 {
     init_members();
 
-    _sock_domain = server_sock->_sock_domain;
-    _sock_type = server_sock->_sock_type;
+    sock_domain = server_sock->sock_domain;
+    sock_type = server_sock->sock_type;
 
     reactor = server_sock->reactor;
     socket = swReactor_get(reactor, _fd);
@@ -344,6 +376,28 @@ Socket::Socket(int _fd, Socket *server_sock)
     socket->fdtype = SW_FD_CORO_SOCKET;
 }
 
+void Socket::set_timer(timer_levels _timer_level, double _timeout)
+{
+    if (_timeout == 0)
+    {
+        _timeout = timeout;
+    }
+    if (!timer && _timeout > 0)
+    {
+        timer_level = _timer_level;
+        timer = swTimer_add(&SwooleG.timer, (long) (_timeout * 1000), 0, this, timer_callback);
+    }
+}
+
+void Socket::del_timer(timer_levels _timer_level)
+{
+    if (timer && _timer_level == timer_level)
+    {
+        swTimer_del(&SwooleG.timer, timer);
+        timer = nullptr;
+    }
+}
+
 bool Socket::connect(const struct sockaddr *addr, socklen_t addrlen)
 {
     if (unlikely(!is_available()))
@@ -353,14 +407,12 @@ bool Socket::connect(const struct sockaddr *addr, socklen_t addrlen)
     int retval = socket_connect(socket->fd, addr, addrlen);
     if (retval == -1)
     {
-        if (errno != EINPROGRESS || !wait_event(SW_EVENT_WRITE))
+        if (errno != EINPROGRESS)
         {
             set_err(errno);
             return false;
         }
-        yield();
-        //Connection has timed out
-        if (errCode == ETIMEDOUT)
+        if (!wait_writeable())
         {
             return false;
         }
@@ -382,7 +434,7 @@ bool Socket::connect(const struct sockaddr *addr, socklen_t addrlen)
     return true;
 }
 
-bool Socket::connect(string host, int port, int flags)
+bool Socket::connect(string _host, int _port, int flags)
 {
     if (unlikely(!is_available()))
     {
@@ -392,55 +444,56 @@ bool Socket::connect(string host, int port, int flags)
     if (socks5_proxy)
     {
         //enable socks5 proxy
-        socks5_proxy->target_host = sw_strndup((char *) host.c_str(), host.size());
-        socks5_proxy->l_target_host = host.size();
-        socks5_proxy->target_port = port;
+        socks5_proxy->target_host = sw_strndup((char *) _host.c_str(), _host.size());
+        socks5_proxy->l_target_host = _host.size();
+        socks5_proxy->target_port = _port;
 
-        host = socks5_proxy->host;
-        port = socks5_proxy->port;
+        _host = socks5_proxy->host;
+        _port = socks5_proxy->port;
     }
     else if (http_proxy)
     {
         //enable http proxy
-        http_proxy->target_host = sw_strndup((char *) host.c_str(), host.size());
-        http_proxy->l_target_host = host.size();
-        http_proxy->target_port = port;
+        http_proxy->target_host = sw_strndup((char *) _host.c_str(), _host.size());
+        http_proxy->l_target_host = _host.size();
+        http_proxy->target_port = _port;
 
-        host = http_proxy->proxy_host;
-        port = http_proxy->proxy_port;
+        _host = http_proxy->proxy_host;
+        _port = http_proxy->proxy_port;
     }
 
-    if (_sock_domain == AF_INET6 || _sock_domain == AF_INET)
+    if (sock_domain == AF_INET6 || sock_domain == AF_INET)
     {
-        if (port == -1)
+        if (_port == -1)
         {
             swWarn("Socket of type AF_INET/AF_INET6 requires port argument");
             return false;
         }
-        else if (port == 0 || port >= 65536)
+        else if (_port == 0 || _port >= 65536)
         {
-            swWarn("Invalid port argument[%d]", port);
+            swWarn("Invalid port argument[%d]", _port);
             return false;
         }
     }
 
-    _host = host;
-    _port = port;
+    host = _host;
+    port = _port;
 
     struct sockaddr *_target_addr = nullptr;
 
     for (int i = 0; i < 2; i++)
     {
-        if (_sock_domain == AF_INET)
+        if (sock_domain == AF_INET)
         {
             socket->info.addr.inet_v4.sin_family = AF_INET;
-            socket->info.addr.inet_v4.sin_port = htons(port);
+            socket->info.addr.inet_v4.sin_port = htons(_port);
 
-            if (!inet_pton(AF_INET, _host.c_str(), & socket->info.addr.inet_v4.sin_addr))
+            if (!inet_pton(AF_INET, host.c_str(), & socket->info.addr.inet_v4.sin_addr))
             {
-                _host = resolve(_host);
-                if (_host.size() == 0)
+                host = Coroutine::gethostbyname(host, AF_INET, timeout);
+                if (host.empty())
                 {
+                    set_err(SwooleG.error);
                     return false;
                 }
                 continue;
@@ -452,16 +505,17 @@ bool Socket::connect(string host, int port, int flags)
                 break;
             }
         }
-        else if (_sock_domain == AF_INET6)
+        else if (sock_domain == AF_INET6)
         {
             socket->info.addr.inet_v6.sin6_family = AF_INET6;
-            socket->info.addr.inet_v6.sin6_port = htons(port);
+            socket->info.addr.inet_v6.sin6_port = htons(_port);
 
-            if (!inet_pton(AF_INET6, _host.c_str(), &socket->info.addr.inet_v6.sin6_addr))
+            if (!inet_pton(AF_INET6, host.c_str(), &socket->info.addr.inet_v6.sin6_addr))
             {
-                _host = resolve(_host);
-                if (_host.size() == 0)
+                host = Coroutine::gethostbyname(host, AF_INET6, timeout);
+                if (host.empty())
                 {
+                    set_err(SwooleG.error);
                     return false;
                 }
                 continue;
@@ -473,15 +527,15 @@ bool Socket::connect(string host, int port, int flags)
                 break;
             }
         }
-        else if (_sock_domain == AF_UNIX)
+        else if (sock_domain == AF_UNIX)
         {
-            if (_host.size() >= sizeof(socket->info.addr.un.sun_path))
+            if (host.size() >= sizeof(socket->info.addr.un.sun_path))
             {
                 return false;
             }
             socket->info.addr.un.sun_family = AF_UNIX;
-            memcpy(&socket->info.addr.un.sun_path, _host.c_str(), _host.size());
-            socket->info.len = (socklen_t) (offsetof(struct sockaddr_un, sun_path) + _host.size());
+            memcpy(&socket->info.addr.un.sun_path, host.c_str(), host.size());
+            socket->info.len = (socklen_t) (offsetof(struct sockaddr_un, sun_path) + host.size());
             _target_addr = (struct sockaddr *) &socket->info.addr.un;
             break;
         }
@@ -513,34 +567,6 @@ bool Socket::connect(string host, int port, int flags)
     return true;
 }
 
-static void socket_dns_callback(swAio_event *event)
-{
-    Socket *sock = (Socket *) event->object;
-    if (event->error != 0)
-    {
-        sock->set_err(event->error, hstrerror(event->error));
-    }
-    sock->resume();
-}
-
-static void socket_timer_callback(swTimer *timer, swTimer_node *tnode)
-{
-    Socket *sock = (Socket *) tnode->data;
-    swTraceLog(SW_TRACE_SOCKET, "socket[%d] timeout", sock->socket->fd);
-    sock->set_err(ETIMEDOUT);
-    sock->reactor->del(sock->reactor, sock->socket->fd);
-    sock->_timer = NULL;
-    sock->resume();
-}
-
-static int socket_event_callback(swReactor *reactor, swEvent *event)
-{
-    Socket *sock = (Socket *) event->socket->object;
-    sock->reactor->del(sock->reactor, event->fd);
-    sock->resume();
-    return SW_OK;
-}
-
 bool Socket::is_connect()
 {
     return socket->active && !socket->closed;
@@ -569,10 +595,6 @@ bool Socket::check_liveness()
 
 ssize_t Socket::peek(void *__buf, size_t __n)
 {
-    if (unlikely(!is_available(true)))
-    {
-        return -1;
-    }
     ssize_t retval = swConnection_peek(socket, __buf, __n, 0);
     set_err(retval < 0 ? errno : 0);
     return retval;
@@ -629,7 +651,7 @@ ssize_t Socket::write(const void *__buf, size_t __n)
         {
             return -1;
         }
-        retval =  ::write(socket->fd, (void *) __buf, __n);
+        retval = ::write(socket->fd, (void *) __buf, __n);
     }
     set_err(retval < 0 ? errno : 0);
     return retval;
@@ -637,11 +659,12 @@ ssize_t Socket::write(const void *__buf, size_t __n)
 
 ssize_t Socket::recv_all(void *__buf, size_t __n)
 {
+    ssize_t retval, total_bytes = 0;
     if (unlikely(!is_available()))
     {
         return -1;
     }
-    ssize_t retval, total_bytes = 0;
+    set_timer(TIMER_LV_MULTI);
     while (true)
     {
         retval = recv((char*) __buf + total_bytes, __n - total_bytes);
@@ -659,16 +682,18 @@ ssize_t Socket::recv_all(void *__buf, size_t __n)
             break;
         }
     }
+    del_timer(TIMER_LV_MULTI);
     return total_bytes;
 }
 
 ssize_t Socket::send_all(const void *__buf, size_t __n)
 {
+    ssize_t retval, total_bytes = 0;
     if (unlikely(!is_available()))
     {
         return -1;
     }
-    ssize_t retval, total_bytes = 0;
+    set_timer(TIMER_LV_MULTI);
     while (true)
     {
         retval = send((char*) __buf + total_bytes, __n - total_bytes);
@@ -686,6 +711,7 @@ ssize_t Socket::send_all(const void *__buf, size_t __n)
             break;
         }
     }
+    del_timer(TIMER_LV_MULTI);
     return total_bytes;
 }
 
@@ -718,14 +744,9 @@ ssize_t Socket::sendmsg(const struct msghdr *msg, int flags)
         return -1;
     }
     ssize_t retval = ::sendmsg(socket->fd, msg, flags);
-    if (retval < 0 && swConnection_error(errno) == SW_WAIT)
+    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
     {
-        if (!wait_event(SW_EVENT_WRITE))
-        {
-            return -1;
-        }
-        yield();
-        if (errCode == ETIMEDOUT)
+        if (!wait_writeable())
         {
             return -1;
         }
@@ -742,14 +763,9 @@ ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
         return -1;
     }
     ssize_t retval = ::recvmsg(socket->fd, msg, flags);
-    if (retval < 0 && swConnection_error(errno) == SW_WAIT)
+    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
     {
-        if (!wait_event(SW_EVENT_READ))
-        {
-            return -1;
-        }
-        yield();
-        if (errCode == ETIMEDOUT)
+        if (!wait_readable())
         {
             return -1;
         }
@@ -759,45 +775,6 @@ ssize_t Socket::recvmsg(struct msghdr *msg, int flags)
     return retval;
 }
 
-void Socket::yield()
-{
-    Coroutine *co = coroutine_get_current();
-    double timeout = _timeout_temp ? _timeout_temp : _timeout;
-
-    if (unlikely(!co))
-    {
-        swError("Socket::yield() must be called in the coroutine.");
-    }
-
-    set_err(0);
-    int ms = (int) (timeout * 1000);
-    if (ms <= 0)
-    {
-        timeout = -1;
-    }
-    if (timeout > 0)
-    {
-        _timer = swTimer_add(&SwooleG.timer, ms, 0, this, socket_timer_callback);
-    }
-
-    //=== bind coroutine ===
-    bind_co = co;
-    //=== yield ===
-    co->yield();
-    //=== resume ===
-    bind_co = nullptr;
-    //=== clear timer ===
-    if (_timer)
-    {
-        swTimer_del(&SwooleG.timer, _timer);
-        _timer = nullptr;
-    }
-    if (_timeout_temp)
-    {
-        _timeout_temp = 0;
-    }
-}
-
 bool Socket::bind(std::string address, int port)
 {
     if (unlikely(!is_available()))
@@ -827,7 +804,7 @@ bool Socket::bind(std::string address, int port)
 #endif
 
     int retval;
-    switch (_sock_domain)
+    switch (sock_domain)
     {
     case AF_UNIX:
     {
@@ -891,12 +868,8 @@ bool Socket::listen(int backlog)
     {
         return false;
     }
-    if (backlog <= 0)
-    {
-        backlog = SW_BACKLOG;
-    }
-    _backlog = backlog;
-    if (::listen(socket->fd, backlog) != 0)
+    this->backlog = backlog <= 0 ? SW_BACKLOG : backlog;
+    if (::listen(socket->fd, this->backlog) != 0)
     {
         set_err(errno);
         return false;
@@ -925,12 +898,7 @@ Socket* Socket::accept()
     int conn = swSocket_accept(socket->fd, &client_addr);
     if (conn < 0 && errno == EAGAIN)
     {
-        if (!wait_event(SW_EVENT_READ))
-        {
-            return nullptr;
-        }
-        yield();
-        if (errCode == ETIMEDOUT)
+        if (!wait_readable())
         {
             return nullptr;
         }
@@ -944,6 +912,7 @@ Socket* Socket::accept()
     Socket *client_sock = new Socket(conn, this);
     if (unlikely(client_sock->socket == nullptr))
     {
+        swWarn("new Socket() failed. Error: %s [%d]", strerror(errno), errno);
         set_err(errno);
         delete client_sock;
         return nullptr;
@@ -966,73 +935,13 @@ Socket* Socket::accept()
     return client_sock;
 }
 
-string Socket::resolve(string domain_name)
-{
-    if (unlikely(!is_available()))
-    {
-        return "";
-    }
-
-    swAio_event ev;
-    bzero(&ev, sizeof(swAio_event));
-    if (domain_name.size() < SW_IP_MAX_LENGTH)
-    {
-        ev.nbytes = SW_IP_MAX_LENGTH + 1;
-    }
-    else
-    {
-        ev.nbytes = domain_name.size() + 1;
-    }
-    ev.buf = sw_malloc(ev.nbytes);
-    if (!ev.buf)
-    {
-        set_err(errno);
-        return "";
-    }
-
-    memcpy(ev.buf, domain_name.c_str(), domain_name.size());
-    ((char *) ev.buf)[domain_name.size()] = 0;
-    ev.flags = _sock_domain;
-    ev.type = SW_AIO_GETHOSTBYNAME;
-    ev.object = this;
-    ev.handler = swAio_handler_gethostbyname;
-    ev.callback = socket_dns_callback;
-
-    if (SwooleAIO.init == 0)
-    {
-        swAio_init();
-    }
-
-    if (swAio_dispatch(&ev) < 0)
-    {
-        set_err(SwooleG.error);
-        sw_free(ev.buf);
-        return "";
-    }
-
-    /** cannot timeout */
-    set_timeout(-1, true);
-    yield();
-
-    if (errCode == SW_ERROR_DNSLOOKUP_RESOLVE_FAILED)
-    {
-        return "";
-    }
-    else
-    {
-        string addr((char *) ev.buf);
-        sw_free(ev.buf);
-        return addr;
-    }
-}
-
 bool Socket::shutdown(int __how)
 {
     if (__how == SHUT_RD && !shutdown_read)
     {
         if (::shutdown(socket->fd, SHUT_RD) == 0)
         {
-            shutdown_read = 1;
+            shutdown_read = true;
             return true;
         }
     }
@@ -1040,7 +949,7 @@ bool Socket::shutdown(int __how)
     {
         if (::shutdown(socket->fd, SHUT_WR) == 0)
         {
-            shutdown_write = 1;
+            shutdown_write = true;
             return true;
         }
     }
@@ -1048,7 +957,7 @@ bool Socket::shutdown(int __how)
     {
         if (::shutdown(socket->fd, SHUT_RDWR) == 0)
         {
-            shutdown_read = shutdown_write = 1;
+            shutdown_read = shutdown_write = true;
             return true;
         }
     }
@@ -1068,7 +977,7 @@ bool Socket::close()
         shutdown();
         socket->active = 0;
     }
-    if (bind_co)
+    if (coroutine)
     {
         reactor->del(reactor, socket->fd);
         resume();
@@ -1224,6 +1133,7 @@ bool Socket::sendfile(char *filename, off_t offset, size_t length)
     }
 
     int n, sendn;
+    set_timer(TIMER_LV_MULTI);
     while ((size_t) offset < length)
     {
         sendn = (length - offset > SW_SENDFILE_CHUNK_SIZE) ? SW_SENDFILE_CHUNK_SIZE : length - offset;
@@ -1250,21 +1160,17 @@ bool Socket::sendfile(char *filename, off_t offset, size_t length)
         else if (errno != EAGAIN)
         {
             swSysError("sendfile(%d, %s) failed.", socket->fd, filename);
-            _error:
             set_err(errno);
             ::close(file_fd);
             return false;
         }
-        if (!wait_event(SW_EVENT_WRITE))
+        if (!wait_writeable())
         {
-            goto _error;
-        }
-        yield();
-        if (errCode == ETIMEDOUT)
-        {
-            goto _error;
+            ::close(file_fd);
+            return false;
         }
     }
+    del_timer(TIMER_LV_MULTI);
     ::close(file_fd);
     return true;
 }
@@ -1311,14 +1217,9 @@ ssize_t Socket::recvfrom(void *__buf, size_t __n, struct sockaddr* _addr, sockle
     {
         retval = ::recvfrom(socket->fd, __buf, __n, 0, _addr, _socklen);
     }
-    if (retval < 0 && swConnection_error(errno) == SW_WAIT)
+    while (retval < 0 && swConnection_error(errno) == SW_WAIT)
     {
-        if (!wait_event(SW_EVENT_READ))
-        {
-            return -1;
-        }
-        yield();
-        if (errCode == ETIMEDOUT)
+        if (!wait_readable())
         {
             return -1;
         }
@@ -1363,7 +1264,8 @@ ssize_t Socket::recv_packet()
             }
         }
 
-        _recv_header: retval = recv(read_buffer->str + read_buffer->length, header_len - read_buffer->length);
+        _recv_header:
+        retval = recv(read_buffer->str + read_buffer->length, header_len - read_buffer->length);
         if (retval <= 0)
         {
             return 0;
@@ -1373,7 +1275,8 @@ ssize_t Socket::recv_packet()
             read_buffer->length += retval;
         }
 
-        _get_length: buf_len = protocol.get_package_length(&protocol, socket, read_buffer->str, (uint32_t) read_buffer->length);
+        _get_length:
+        buf_len = protocol.get_package_length(&protocol, socket, read_buffer->str, (uint32_t) read_buffer->length);
         swTraceLog(SW_TRACE_SOCKET, "packet_len=%ld, length=%ld", buf_len, read_buffer->length);
         //error package
         if (buf_len < 0)
@@ -1526,14 +1429,28 @@ ssize_t Socket::recv_packet()
 
 Socket::~Socket()
 {
+    int fd;
     if (socket == nullptr)
     {
+        // construct failed
         return;
     }
-    if (_sock_domain == AF_UNIX && bind_address.size() > 0)
+    if (read_buffer)
+    {
+        swString_free(read_buffer);
+    }
+    if (write_buffer)
+    {
+        swString_free(write_buffer);
+    }
+    if (sock_domain == AF_UNIX && !bind_address.empty())
     {
         unlink(bind_address_info.addr.un.sun_path);
     }
+    if (sock_type == SW_SOCK_UNIX_DGRAM)
+    {
+        unlink(socket->info.addr.un.sun_path);
+    }
 #ifdef SW_USE_OPENSSL
     if (socket->ssl)
     {
@@ -1568,35 +1485,23 @@ Socket::~Socket()
         {
             sw_free(ssl_option.capath);
         }
-        ssl_option = {0};
-        ssl_context = nullptr;
     }
 #endif
-    if (_sock_type == SW_SOCK_UNIX_DGRAM)
-    {
-        unlink(socket->info.addr.un.sun_path);
-    }
-
-    if (socket->out_buffer)
-    {
-        swBuffer_free(socket->out_buffer);
-        socket->out_buffer = NULL;
-    }
     if (socket->in_buffer)
     {
         swBuffer_free(socket->in_buffer);
-        socket->in_buffer = NULL;
     }
-    if (read_buffer)
+    if (socket->out_buffer)
     {
-        swString_free(read_buffer);
+        swBuffer_free(socket->out_buffer);
     }
-    int fd = socket->fd;
+    fd = socket->fd;
     if (socket->removed == 0)
     {
         reactor->del(reactor, fd);
     }
     bzero(socket, sizeof(swConnection));
     socket->removed = 1;
+    socket->closed = 1;
     ::close(fd);
 }
