@@ -21,14 +21,14 @@ static swEventData *g_current_task = NULL;
 
 static void swTaskWorker_signal_init(swProcessPool *pool);
 static int swTaskWorker_onPipeReceive(swReactor *reactor, swEvent *event);
-static int swTaskWorker_loop_async(struct _swProcessPool *pool, swWorker *worker);
+static int swTaskWorker_loop_async(swProcessPool *pool, swWorker *worker);
 
 /**
  * after pool->create, before pool->start
  */
-void swTaskWorker_init(swProcessPool *pool)
+void swTaskWorker_init(swServer *serv)
 {
-    swServer *serv = SwooleG.serv;
+    swProcessPool *pool = &serv->gs->task_workers;
     pool->ptr = serv;
     pool->onTask = swTaskWorker_onTask;
     pool->onWorkerStart = swTaskWorker_onStart;
@@ -38,7 +38,7 @@ void swTaskWorker_init(swProcessPool *pool)
     /**
      * Make the task worker support asynchronous
      */
-    if (serv->task_async)
+    if (serv->task_enable_coroutine)
     {
         pool->main_loop = swTaskWorker_loop_async;
     }
@@ -147,7 +147,7 @@ void swTaskWorker_onStart(swProcessPool *pool, int worker_id)
     /**
      * Make the task worker support asynchronous
      */
-    if (serv->task_async)
+    if (serv->task_enable_coroutine)
     {
         SwooleG.main_reactor = sw_malloc(sizeof(swReactor));
         if (SwooleG.main_reactor == NULL)
