@@ -412,6 +412,11 @@ void swWorker_onStart(swServer *serv)
         SwooleG.process_type = SW_PROCESS_WORKER;
     }
 
+    if (serv->enable_coroutine)
+    {
+        SwooleG.enable_coroutine = 1;
+    }
+
     SwooleG.memory_pool = swMemoryGlobal_new(SW_GLOBAL_MEMORY_PAGESIZE, 1);
     if (SwooleG.memory_pool == NULL)
     {
@@ -574,8 +579,9 @@ void swWorker_stop(swWorker *worker)
         kill(serv->gs->manager_pid, SIGIO);
     }
 
-    try_to_exit: SwooleWG.wait_exit = 1;
-    swTimer_add(&SwooleG.timer, serv->max_wait_time * 1000, 0, NULL, swWorker_onTimeout);
+    try_to_exit:
+    SwooleWG.wait_exit = 1;
+    swTimer_add(&SwooleG.timer, (long) (serv->max_wait_time * 1000), 0, NULL, swWorker_onTimeout);
 
     swWorker_try_to_exit();
 }
