@@ -28,13 +28,15 @@
 #include <string>
 #include <unordered_map>
 
+using namespace swoole;
+
 typedef struct
 {
     zval _callback;
     zval _filename;
     zval *callback;
     zval *filename;
-    int fd;
+    uint32_t *refcount;
     off_t offset;
     uint16_t type;
     uint8_t once;
@@ -48,7 +50,7 @@ typedef struct
     zval _domain;
     zval *callback;
     zval *domain;
-    php_context *context;
+    php_coro_context *context;
     uint8_t useless;
     swTimer_node *timer;
 } dns_request;
@@ -77,7 +79,12 @@ static void dns_timeout_coro(swTimer *timer, swTimer_node *tnode);
 
 static void php_swoole_file_request_free(void *data);
 
-static std::unordered_map<std::string, int> open_files;
+typedef struct
+{
+    int fd;
+    uint32_t refcount;
+} open_file;
+static std::unordered_map<std::string, open_file> open_write_files;
 static std::unordered_map<std::string, dns_cache*> request_cache_map;
 
 static void php_swoole_file_request_free(void *data)
@@ -95,18 +102,8 @@ static void php_swoole_file_request_free(void *data)
 void swoole_async_init(int module_number)
 {
     bzero(&SwooleAIO, sizeof(SwooleAIO));
-
-    REGISTER_LONG_CONSTANT("SWOOLE_AIO_BASE", 0, CONST_CS | CONST_PERSISTENT);
-    REGISTER_LONG_CONSTANT("SWOOLE_AIO_LINUX", 0, CONST_CS | CONST_PERSISTENT);
-}
-
-void php_swoole_check_aio()
-{
-    if (unlikely(SwooleAIO.init == 0))
-    {
-        php_swoole_check_reactor();
-        swAio_init();
-    }
+    SwooleAIO.min_thread_count = SW_AIO_THREAD_MIN_NUM;
+    SwooleAIO.max_thread_count = SW_AIO_THREAD_MAX_NUM;
 }
 
 static void php_swoole_dns_callback(char *domain, swDNSResolver_result *result, void *data)
@@ -216,7 +213,7 @@ static void coro_onDNSCompleted(char *domain, swDNSResolver_result *result, void
         return;
     }
 
-    int ret = sw_coro_resume(req->context, zaddress, retval);
+    int ret = PHPCoroutine::resume_m(req->context, zaddress, retval);
     if (ret > 0)
     {
         goto free_zdata;
@@ -236,7 +233,7 @@ static void dns_timeout_coro(swTimer *timer, swTimer_node *tnode)
 {
     zval *retval = NULL;
     zval *zaddress;
-    php_context *cxt = (php_context *) tnode->data;
+    php_coro_context *cxt = (php_coro_context *) tnode->data;
     dns_request *req = (dns_request *) cxt->coro_params.value.ptr;
 
     SW_MAKE_STD_ZVAL(zaddress);
@@ -251,7 +248,7 @@ static void dns_timeout_coro(swTimer *timer, swTimer_node *tnode)
         ZVAL_STRING(zaddress, "");
     }
 
-    int ret = sw_coro_resume(req->context, zaddress, retval);
+    int ret = PHPCoroutine::resume_m(req->context, zaddress, retval);
     if (ret > 0)
     {
         goto free_zdata;
@@ -402,14 +399,29 @@ static void aio_onFileCompleted(swAio_event *event)
     if (file_req->once == 1)
     {
         close_file:
-        close(event->fd);
+        if (file_req->refcount)
+        {
+            if (--(*file_req->refcount) == 0)
+            {
+                swTraceLog(SW_TRACE_AIO, "close file fd#%d", event->fd);
+                open_write_files.erase(std::string(Z_STRVAL_P(file_req->filename), Z_STRLEN_P(file_req->filename)));
+                close(event->fd);
+            }
+            else
+            {
+                swTraceLog(SW_TRACE_AIO, "delref file fd#%d, refcount=%u", event->fd, *file_req->refcount);
+            }
+        }
+        else
+        {
+            close(event->fd);
+        }
         php_swoole_file_request_free(file_req);
     }
     else if(file_req->type == SW_AIO_WRITE)
     {
         if (retval && !ZVAL_IS_NULL(retval) && !Z_BVAL_P(retval))
         {
-            open_files.erase(std::string(Z_STRVAL_P(file_req->filename), Z_STRLEN_P(file_req->filename)));
             goto close_file;
         }
         else
@@ -417,7 +429,7 @@ static void aio_onFileCompleted(swAio_event *event)
             php_swoole_file_request_free(file_req);
         }
     }
-    else
+    else // if(file_req->type == SW_AIO_READ)
     {
         if ((retval && !ZVAL_IS_NULL(retval) && !Z_BVAL_P(retval)) || isEOF)
         {
@@ -433,6 +445,7 @@ static void aio_onFileCompleted(swAio_event *event)
         else
         {
             swAio_event ev;
+            ev.canceled = 0;
             ev.fd = event->fd;
             ev.buf = event->buf;
             ev.type = SW_AIO_READ;
@@ -468,10 +481,10 @@ static void aio_onFileCompleted(swAio_event *event)
 
 PHP_FUNCTION(swoole_async_read)
 {
-    zval *callback;
     zval *filename;
-    long buf_size = SW_AIO_DEFAULT_CHUNK_SIZE;
-    long offset = 0;
+    zval *callback;
+    zend_long buf_size = SW_AIO_DEFAULT_CHUNK_SIZE;
+    zend_long offset = 0;
     int open_flag = O_RDONLY;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "zz|ll", &filename, &callback, &buf_size, &offset) == FAILURE)
@@ -524,7 +537,6 @@ PHP_FUNCTION(swoole_async_read)
     }
 
     file_request *req = (file_request *) emalloc(sizeof(file_request));
-    req->fd = fd;
 
     req->filename = filename;
     Z_TRY_ADDREF_P(filename);
@@ -538,15 +550,15 @@ PHP_FUNCTION(swoole_async_read)
     req->callback = callback;
     Z_TRY_ADDREF_P(callback);
     sw_copy_to_stack(req->callback, req->_callback);
+    req->refcount = nullptr;
     req->content = (char*) fcnt;
     req->once = 0;
     req->type = SW_AIO_READ;
     req->length = buf_size;
     req->offset = offset;
 
-    php_swoole_check_aio();
-
     swAio_event ev;
+    ev.canceled = 0;
     ev.fd = fd;
     ev.buf = fcnt;
     ev.type = SW_AIO_READ;
@@ -557,6 +569,7 @@ PHP_FUNCTION(swoole_async_read)
     ev.handler = swAio_handler_read;
     ev.callback = aio_onFileCompleted;
 
+    php_swoole_check_reactor();
     int ret = swAio_dispatch(&ev);
     if (ret == SW_ERR)
     {
@@ -570,18 +583,17 @@ PHP_FUNCTION(swoole_async_read)
 
 PHP_FUNCTION(swoole_async_write)
 {
-    zval *callback = NULL;
     zval *filename;
-
     char *fcnt;
-    size_t fcnt_len = 0;
+    size_t fcnt_len;
     off_t offset = -1;
+    zval *callback = NULL;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "zs|lz", &filename, &fcnt, &fcnt_len, &offset, &callback) == FAILURE)
     {
         RETURN_FALSE;
     }
-    if (fcnt_len <= 0)
+    if (fcnt_len == 0)
     {
         RETURN_FALSE;
     }
@@ -595,10 +607,17 @@ PHP_FUNCTION(swoole_async_write)
 
     convert_to_string(filename);
 
+    if (offset < 0)
+    {
+        offset = 0;
+    }
+
+    file_request *req = (file_request *) emalloc(sizeof(file_request));
+
     int fd;
     std::string key(Z_STRVAL_P(filename), Z_STRLEN_P(filename));
-    auto file_iterator = open_files.find(key);
-    if (file_iterator == open_files.end())
+    auto file_iterator = open_write_files.find(key);
+    if (file_iterator == open_write_files.end())
     {
         int open_flag = O_WRONLY | O_CREAT;
         if (offset < 0)
@@ -611,21 +630,19 @@ PHP_FUNCTION(swoole_async_write)
             swoole_php_fatal_error(E_WARNING, "open(%s, %d) failed. Error: %s[%d]", Z_STRVAL_P(filename), open_flag, strerror(errno), errno);
             RETURN_FALSE;
         }
-        open_files[key] = fd;
+        swTraceLog(SW_TRACE_AIO, "open write file fd#%d", fd);
+        open_write_files[key] = {fd, 1};
+        req->refcount = &open_write_files[key].refcount;
     }
     else
     {
-        fd = file_iterator->second;
+        fd = file_iterator->second.fd;
+        file_iterator->second.refcount++;
+        req->refcount = &file_iterator->second.refcount;
+        swTraceLog(SW_TRACE_AIO, "reuse write file fd#%d", fd);
     }
 
-    if (offset < 0)
-    {
-        offset = 0;
-    }
-
-    file_request *req = (file_request *) emalloc(sizeof(file_request));
     char *wt_cnt = (char *) emalloc(fcnt_len);
-    req->fd = fd;
     req->content = wt_cnt;
     req->once = 0;
     req->type = SW_AIO_WRITE;
@@ -647,9 +664,9 @@ PHP_FUNCTION(swoole_async_write)
     }
 
     memcpy(wt_cnt, fcnt, fcnt_len);
-    php_swoole_check_aio();
 
     swAio_event ev;
+    ev.canceled = 0;
     ev.fd = fd;
     ev.buf = wt_cnt;
     ev.type = SW_AIO_WRITE;
@@ -660,6 +677,7 @@ PHP_FUNCTION(swoole_async_write)
     ev.handler = swAio_handler_write;
     ev.callback = aio_onFileCompleted;
 
+    php_swoole_check_reactor();
     int ret = swAio_dispatch(&ev);
     if (ret == SW_ERR)
     {
@@ -718,7 +736,6 @@ PHP_FUNCTION(swoole_async_readfile)
 
     size_t length = file_stat.st_size;
     file_request *req = (file_request *) emalloc(sizeof(file_request));
-    req->fd = fd;
 
     req->filename = filename;
     Z_TRY_ADDREF_P(filename);
@@ -727,16 +744,15 @@ PHP_FUNCTION(swoole_async_readfile)
     req->callback = callback;
     Z_TRY_ADDREF_P(callback);
     sw_copy_to_stack(req->callback, req->_callback);
-
+    req->refcount = nullptr;
     req->content = (char *) emalloc(length);
     req->once = 1;
     req->type = SW_AIO_READ;
     req->length = length;
     req->offset = 0;
 
-    php_swoole_check_aio();
-
     swAio_event ev;
+    ev.canceled = 0;
     ev.fd = fd;
     ev.buf = req->content;
     ev.type = SW_AIO_READ;
@@ -747,6 +763,7 @@ PHP_FUNCTION(swoole_async_readfile)
     ev.handler = swAio_handler_read;
     ev.callback = aio_onFileCompleted;
 
+    php_swoole_check_reactor();
     int ret = swAio_dispatch(&ev);
     if (ret == SW_ERR)
     {
@@ -760,11 +777,11 @@ PHP_FUNCTION(swoole_async_readfile)
 
 PHP_FUNCTION(swoole_async_writefile)
 {
-    zval *callback = NULL;
     zval *filename;
     char *fcnt;
     size_t fcnt_len;
-    long flags = 0;
+    zval *callback = NULL;
+    zend_long flags = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "zs|zl", &filename, &fcnt, &fcnt_len, &callback, &flags) == FAILURE)
     {
@@ -779,7 +796,7 @@ PHP_FUNCTION(swoole_async_writefile)
     {
         open_flag |= O_TRUNC;
     }
-    if (fcnt_len <= 0)
+    if (fcnt_len == 0)
     {
         RETURN_FALSE;
     }
@@ -823,8 +840,7 @@ PHP_FUNCTION(swoole_async_writefile)
     {
         req->callback = NULL;
     }
-
-    req->fd = fd;
+    req->refcount = nullptr;
     req->type = SW_AIO_WRITE;
     req->content = wt_cnt;
     req->once = 1;
@@ -833,9 +849,8 @@ PHP_FUNCTION(swoole_async_writefile)
 
     memcpy(wt_cnt, fcnt, fcnt_len);
 
-    php_swoole_check_aio();
-
     swAio_event ev;
+    ev.canceled = 0;
     ev.fd = fd;
     ev.buf = wt_cnt;
     ev.type = SW_AIO_WRITE;
@@ -846,6 +861,7 @@ PHP_FUNCTION(swoole_async_writefile)
     ev.handler = swAio_handler_write;
     ev.callback = aio_onFileCompleted;
 
+    php_swoole_check_reactor();
     int ret = swAio_dispatch(&ev);
     if (ret == SW_ERR)
     {
@@ -876,11 +892,6 @@ PHP_FUNCTION(swoole_async_set)
     php_swoole_array_separate(zset);
 
     vht = Z_ARRVAL_P(zset);
-    if (php_swoole_array_get_value(vht, "thread_num", v))
-    {
-        convert_to_long(v);
-        SwooleAIO.thread_num = (uint8_t) Z_LVAL_P(v);
-    }
     if (php_swoole_array_get_value(vht, "enable_signalfd", v))
     {
         convert_to_boolean(v);
@@ -907,6 +918,16 @@ PHP_FUNCTION(swoole_async_set)
         level = Z_LVAL_P(v);
         SwooleG.log_level = (uint32_t) (level < 0 ? UINT32_MAX : level);
     }
+    if (php_swoole_array_get_value(vht, "thread_num", v) || php_swoole_array_get_value(vht, "min_thread_num", v))
+    {
+        convert_to_long(v);
+        SwooleAIO.max_thread_count = SwooleAIO.min_thread_count = Z_LVAL_P(v);
+    }
+    if (php_swoole_array_get_value(vht, "max_thread_num", v))
+    {
+        convert_to_long(v);
+        SwooleAIO.max_thread_count = Z_LVAL_P(v);
+    }
     if (php_swoole_array_get_value(vht, "display_errors", v))
     {
         convert_to_boolean(v);
@@ -996,8 +1017,6 @@ PHP_FUNCTION(swoole_async_dns_lookup)
         SW_CHECK_RETURN(swDNSResolver_request(Z_STRVAL_P(domain), php_swoole_dns_callback, (void *) req));
     }
 
-    php_swoole_check_aio();
-
     /**
      * Use thread pool
      */
@@ -1014,9 +1033,9 @@ PHP_FUNCTION(swoole_async_dns_lookup)
     void *buf = emalloc(buf_size);
     bzero(buf, buf_size);
     memcpy(buf, Z_STRVAL_P(domain), Z_STRLEN_P(domain));
-    php_swoole_check_aio();
 
     swAio_event ev;
+    ev.canceled = 0;
     ev.fd = 0;
     ev.buf = buf;
     ev.type = SW_AIO_WRITE;
@@ -1027,6 +1046,8 @@ PHP_FUNCTION(swoole_async_dns_lookup)
     ev.req = req;
     ev.handler = swAio_handler_gethostbyname;
     ev.callback = aio_onDNSCompleted;
+
+    php_swoole_check_reactor();
     SW_CHECK_RETURN(swAio_dispatch(&ev));
 }
 
@@ -1169,12 +1190,12 @@ PHP_METHOD(swoole_async, exec)
 PHP_FUNCTION(swoole_async_dns_lookup_coro)
 {
     zval *domain;
-    double timeout = COROG.socket_connect_timeout;
+    double timeout = PHPCoroutine::socket_connect_timeout;
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|d", &domain, &timeout) == FAILURE)
     {
         RETURN_FALSE;
     }
-    coro_check();
+    PHPCoroutine::check();
     if (Z_TYPE_P(domain) != IS_STRING)
     {
         swoole_php_fatal_error(E_WARNING, "invalid domain name.");
@@ -1205,7 +1226,7 @@ PHP_FUNCTION(swoole_async_dns_lookup_coro)
     sw_copy_to_stack(req->domain, req->_domain);
     req->useless = 0;
 
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
+    php_coro_context *context = (php_coro_context *) emalloc(sizeof(php_coro_context));
     context->state = SW_CORO_CONTEXT_RUNNING;
     context->coro_params.value.ptr = (void *) req;
     req->context = context;
@@ -1217,12 +1238,11 @@ PHP_FUNCTION(swoole_async_dns_lookup_coro)
         SW_CHECK_RETURN(ret);
     }
     //add timeout
-    req->timer = swTimer_add(&SwooleG.timer, (int) (timeout * 1000), 0, context, dns_timeout_coro);
+    req->timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, context, dns_timeout_coro);
     if (req->timer)
     {
         context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
     }
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, context);
 }
 #endif
