@@ -15,7 +15,6 @@
 */
 
 #include "php_swoole.h"
-#include "socket.h"
 #include "swoole_coroutine.h"
 #include "socks5.h"
 #include "mqtt.h"
@@ -102,10 +101,9 @@ static PHP_METHOD(swoole_client_coro, getsockname);
 static PHP_METHOD(swoole_client_coro, getpeername);
 static PHP_METHOD(swoole_client_coro, close);
 
-static void client_coro_check_ssl_setting(Socket *cli, zval *zset);
+static void sw_coro_socket_set_ssl(Socket *cli, zval *zset);
 static Socket* client_coro_new(zval *zobject, int port = 0);
 bool php_swoole_client_coro_socket_free(Socket *cli);
-void php_swoole_client_coro_check_setting(Socket *cli, zval *zset);
 
 static const zend_function_entry swoole_client_coro_methods[] =
 {
@@ -157,7 +155,7 @@ void swoole_client_coro_init(int module_number)
     zend_declare_class_constant_long(swoole_client_coro_ce_ptr, ZEND_STRL("MSG_WAITALL"), MSG_WAITALL);
 }
 
-static sw_inline Socket* client_get_ptr(zval *zobject)
+static sw_inline Socket* client_get_ptr(zval *zobject, bool silent = false)
 {
     Socket *cli = (Socket *) swoole_get_object(zobject);
     if (cli && cli->socket && cli->socket->active == 1)
@@ -166,10 +164,13 @@ static sw_inline Socket* client_get_ptr(zval *zobject)
     }
     else
     {
-        SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
-        zend_update_property_long(swoole_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
-        swoole_php_error(E_WARNING, "client is not connected to server.");
-        return NULL;
+        if (!silent)
+        {
+            SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
+            zend_update_property_long(swoole_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
+            swoole_php_error(E_WARNING, "client is not connected to server.");
+        }
+        return nullptr;
     }
 }
 
@@ -195,9 +196,9 @@ static Socket* client_coro_new(zval *zobject, int port)
     Socket *cli = new Socket((enum swSocket_type) type);
     if (unlikely(cli->socket == nullptr))
     {
-        delete cli;
         swoole_php_fatal_error(E_WARNING, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
         zend_update_property_long(Z_OBJCE_P(zobject), zobject, ZEND_STRL("errCode"), errno);
+        delete cli;
         return NULL;
     }
 
@@ -215,6 +216,29 @@ static Socket* client_coro_new(zval *zobject, int port)
     return cli;
 }
 
+static bool client_coro_close(zval *zobject)
+{
+    Socket *cli = (Socket *) swoole_get_object(zobject);
+
+    zend_update_property_bool(Z_OBJCE_P(zobject), zobject, ZEND_STRL("connected"), 0);
+
+    if (!cli)
+    {
+        return false;
+    }
+#ifdef SWOOLE_SOCKETS_SUPPORT
+    zval *zsocket = (zval *) swoole_get_property(zobject, client_property_socket);
+    if (zsocket)
+    {
+        sw_zval_free(zsocket);
+        swoole_set_property(zobject, client_property_socket, NULL);
+    }
+#endif
+    swoole_set_object(zobject, NULL);
+
+    return php_swoole_client_coro_socket_free(cli);
+}
+
 bool php_swoole_client_coro_socket_free(Socket *cli)
 {
     bool ret;
@@ -267,7 +291,7 @@ bool php_swoole_client_coro_socket_free(Socket *cli)
     return ret;
 }
 
-void php_swoole_client_coro_check_setting(Socket *cli, zval *zset)
+void sw_coro_socket_set(Socket *cli, zval *zset)
 {
     HashTable *vht;
     zval *v;
@@ -529,13 +553,13 @@ void php_swoole_client_coro_check_setting(Socket *cli, zval *zset)
 #ifdef SW_USE_OPENSSL
     if (cli->open_ssl)
     {
-        client_coro_check_ssl_setting(cli, zset);
+        sw_coro_socket_set_ssl(cli, zset);
     }
 #endif
 }
 
 #ifdef SW_USE_OPENSSL
-static void client_coro_check_ssl_setting(Socket *cli, zval *zset)
+static void sw_coro_socket_set_ssl(Socket *cli, zval *zset)
 {
     HashTable *vht = Z_ARRVAL_P(zset);
     zval *v;
@@ -617,7 +641,7 @@ static void client_coro_check_ssl_setting(Socket *cli, zval *zset)
 
 static PHP_METHOD(swoole_client_coro, __construct)
 {
-    long type = 0;
+    zend_long type = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|ls", &type) == FAILURE)
     {
@@ -647,44 +671,34 @@ static PHP_METHOD(swoole_client_coro, __destruct)
 {
     SW_PREVENT_USER_DESTRUCT;
 
-    Socket *cli = (Socket *) swoole_get_object(getThis());
-    //no keep connection
-    if (cli)
-    {
-        zval *zobject = getThis();
-        zval *retval = NULL;
-        sw_zend_call_method_with_0_params(&zobject, swoole_client_coro_ce_ptr, NULL, "close", &retval);
-        if (retval)
-        {
-            zval_ptr_dtor(retval);
-        }
-    }
+    client_coro_close(getThis());
 }
 
 static PHP_METHOD(swoole_client_coro, set)
 {
-    zval *zset;
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &zset) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
-    if (Z_TYPE_P(zset) != IS_ARRAY)
-    {
-        RETURN_FALSE;
-    }
+    Socket *cli = client_get_ptr(getThis(), true);
+    zval *zset, *zsetting;
 
-    zval *zsetting = sw_zend_read_property_array(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_ARRAY(zset)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    zsetting = sw_zend_read_property_array(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
     php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
-
+    if (cli)
+    {
+        sw_coro_socket_set(cli, zset);
+    }
     RETURN_TRUE;
 }
 
 static PHP_METHOD(swoole_client_coro, connect)
 {
-    zend_long port = 0, sock_flag = 0;
-    char *host = NULL;
+    char *host;
     size_t host_len;
-    double timeout = COROG.socket_connect_timeout;
+    zend_long port = 0;
+    double timeout = 0;
+    zend_long sock_flag = 0;
 
     ZEND_PARSE_PARAMETERS_START(1, 4)
         Z_PARAM_STRING(host, host_len)
@@ -716,26 +730,31 @@ static PHP_METHOD(swoole_client_coro, connect)
     zval *zset = sw_zend_read_property(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
     if (zset && ZVAL_IS_ARRAY(zset))
     {
-        php_swoole_client_coro_check_setting(cli, zset);
+        sw_coro_socket_set(cli, zset);
     }
 
-    sw_coro_check_bind("client", cli->has_bound());
-    cli->set_timeout(timeout, true);
+    PHPCoroutine::check_bind("client", cli->has_bound());
+    cli->set_timeout(timeout == 0 ? PHPCoroutine::socket_connect_timeout : timeout);
     if (!cli->connect(host, port, sock_flag))
     {
-        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), cli->errCode);
         swoole_php_error(E_WARNING, "connect to server[%s:%d] failed. Error: %s[%d]", host, (int )port, cli->errMsg, cli->errCode);
+        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), cli->errCode);
+        client_coro_close(getThis());
         RETURN_FALSE;
     }
-    zend_update_property_bool(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("connected"), 1);
-    RETURN_TRUE;
+    else
+    {
+        cli->set_timeout(timeout == 0 ? PHPCoroutine::socket_timeout : timeout);
+        zend_update_property_bool(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("connected"), 1);
+        RETURN_TRUE;
+    }
 }
 
 static PHP_METHOD(swoole_client_coro, send)
 {
     char *data;
     size_t data_len;
-    double timeout = COROG.socket_timeout;
+    double timeout = 0;
 
     ZEND_PARSE_PARAMETERS_START(1, 2)
         Z_PARAM_STRING(data, data_len)
@@ -743,7 +762,7 @@ static PHP_METHOD(swoole_client_coro, send)
         Z_PARAM_DOUBLE(timeout)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    if (data_len <= 0)
+    if (data_len == 0)
     {
         swoole_php_fatal_error(E_WARNING, "data to send is empty.");
         RETURN_FALSE;
@@ -757,9 +776,11 @@ static PHP_METHOD(swoole_client_coro, send)
 
     //clear errno
     SwooleG.error = 0;
-    sw_coro_check_bind("client", cli->has_bound());
-    cli->set_timeout(timeout, true);
+    PHPCoroutine::check_bind("client", cli->has_bound());
+    double persistent_timeout = cli->get_timeout();
+    cli->set_timeout(timeout);
     int ret = cli->send_all(data, data_len);
+    cli->set_timeout(persistent_timeout);
     if (ret < 0)
     {
         swoole_php_sys_error(E_WARNING, "failed to send(%d) %zd bytes.", cli->socket->fd, data_len);
@@ -785,7 +806,7 @@ static PHP_METHOD(swoole_client_coro, sendto)
         RETURN_FALSE;
     }
 
-    if (len <= 0)
+    if (len == 0)
     {
         swoole_php_error(E_WARNING, "data to send is empty.");
         RETURN_FALSE;
@@ -801,7 +822,7 @@ static PHP_METHOD(swoole_client_coro, sendto)
         }
         cli->socket->active = 1;
     }
-    sw_coro_check_bind("client", cli->has_bound());
+    PHPCoroutine::check_bind("client", cli->has_bound());
     SW_CHECK_RETURN(cli->sendto(ip, port, data, len));
 }
 
@@ -833,7 +854,7 @@ static PHP_METHOD(swoole_client_coro, recvfrom)
     }
 
     zend_string *retval = zend_string_alloc(length + 1, 0);
-    sw_coro_check_bind("client", cli->has_bound());
+    PHPCoroutine::check_bind("client", cli->has_bound());
     // cli->set_timeout(timeout, true); TODO
     ssize_t n_bytes = cli->recvfrom(retval->val, length);
     if (n_bytes < 0)
@@ -846,7 +867,10 @@ static PHP_METHOD(swoole_client_coro, recvfrom)
         ZSTR_LEN(retval) = n_bytes;
         ZSTR_VAL(retval)[ZSTR_LEN(retval)] = '\0';
         ZVAL_STRING(address, swConnection_get_ip(cli->socket));
-        ZVAL_LONG(port, swConnection_get_port(cli->socket));
+        if (port)
+        {
+            ZVAL_LONG(port, swConnection_get_port(cli->socket));
+        }
         RETURN_STR(retval);
     }
 }
@@ -855,14 +879,14 @@ static PHP_METHOD(swoole_client_coro, sendfile)
 {
     char *file;
     size_t file_len;
-    long offset = 0;
-    long length = 0;
+    zend_long offset = 0;
+    zend_long length = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|ll", &file, &file_len, &offset, &length) == FAILURE)
     {
         RETURN_FALSE;
     }
-    if (file_len <= 0)
+    if (file_len == 0)
     {
         swoole_php_fatal_error(E_WARNING, "file to send is empty.");
         RETURN_FALSE;
@@ -881,7 +905,7 @@ static PHP_METHOD(swoole_client_coro, sendfile)
     }
     //clear errno
     SwooleG.error = 0;
-    sw_coro_check_bind("client", cli->has_bound());
+    PHPCoroutine::check_bind("client", cli->has_bound());
     int ret = cli->sendfile(file, offset, length);
     if (ret < 0)
     {
@@ -898,7 +922,7 @@ static PHP_METHOD(swoole_client_coro, sendfile)
 
 static PHP_METHOD(swoole_client_coro, recv)
 {
-    double timeout = COROG.socket_timeout;
+    double timeout = 0;
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
         Z_PARAM_OPTIONAL
@@ -910,12 +934,13 @@ static PHP_METHOD(swoole_client_coro, recv)
     {
         RETURN_FALSE;
     }
-    sw_coro_check_bind("client", cli->has_bound());
-    cli->set_timeout(timeout, true);
+    PHPCoroutine::check_bind("client", cli->has_bound());
     ssize_t retval ;
     if (cli->open_length_check || cli->open_eof_check)
     {
+        cli->set_timer(Socket::TIMER_LV_GLOBAL, timeout);
         retval = cli->recv_packet();
+        cli->del_timer(Socket::TIMER_LV_GLOBAL);
         if (retval > 0)
         {
             RETVAL_STRINGL(cli->read_buffer->str, retval);
@@ -924,7 +949,10 @@ static PHP_METHOD(swoole_client_coro, recv)
     else
     {
         zend_string *result = zend_string_alloc(SW_PHP_CLIENT_BUFFER_SIZE - sizeof(zend_string), 0);
+        double persistent_timeout = cli->get_timeout();
+        cli->set_timeout(timeout);
         retval = cli->recv(result->val, SW_PHP_CLIENT_BUFFER_SIZE - sizeof(zend_string));
+        cli->set_timeout(persistent_timeout);
         if (retval > 0)
         {
             result->val[retval] = 0;
@@ -1104,30 +1132,7 @@ static PHP_METHOD(swoole_client_coro, getpeername)
 
 static PHP_METHOD(swoole_client_coro, close)
 {
-    zval *zobject = getThis();
-    Socket *cli = (Socket *) swoole_get_object(zobject);
-
-    zend_update_property_bool(Z_OBJCE_P(zobject), zobject, ZEND_STRL("connected"), 0);
-
-    if (!cli)
-    {
-        swoole_php_fatal_error(E_WARNING, "client is not connected to the server.");
-        RETURN_FALSE;
-    }
-
-#ifdef SWOOLE_SOCKETS_SUPPORT
-    zval *zsocket = (zval *) swoole_get_property(zobject, client_property_socket);
-    if (zsocket)
-    {
-        sw_zval_free(zsocket);
-        swoole_set_property(zobject, client_property_socket, NULL);
-    }
-#endif
-
-    int ret = php_swoole_client_coro_socket_free(cli) ? SW_OK : SW_ERR;
-    swoole_set_object(zobject, NULL);
-
-    SW_CHECK_RETURN(ret);
+    RETURN_BOOL(client_coro_close(getThis()));
 }
 
 #ifdef SW_USE_OPENSSL
@@ -1152,9 +1157,9 @@ static PHP_METHOD(swoole_client_coro, enableSSL)
     zval *zset = sw_zend_read_property(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 0);
     if (ZVAL_IS_ARRAY(zset))
     {
-        client_coro_check_ssl_setting(cli, zset);
+        sw_coro_socket_set_ssl(cli, zset);
     }
-    sw_coro_check_bind("client", cli->has_bound());
+    PHPCoroutine::check_bind("client", cli->has_bound());
     if (cli->ssl_handshake() == false)
     {
         RETURN_FALSE;
