@@ -89,8 +89,8 @@
 
 #define SW_TASKWAIT_TIMEOUT        0.5
 
-#define SW_AIO_THREAD_NUM_DEFAULT        2
-#define SW_AIO_THREAD_NUM_MAX            32
+#define SW_AIO_THREAD_MIN_NUM            4
+#define SW_AIO_THREAD_MAX_NUM            1024
 #define SW_AIO_MAX_FILESIZE              (4*1024*1024)
 #define SW_AIO_EVENT_NUM                 128
 #define SW_AIO_DEFAULT_CHUNK_SIZE        65536
@@ -166,8 +166,6 @@
 
 #define SW_TABLE_CONFLICT_PROPORTION     0.2 //20%
 #define SW_TABLE_KEY_SIZE                64
-//#define SW_TABLE_USE_PHP_HASH
-//#define SW_TABLE_DEBUG
 #define SW_TABLE_USE_SPINLOCK            1
 
 #define SW_SSL_BUFFER_SIZE               16384
@@ -253,12 +251,13 @@
 /**
  * Coroutine
  */
-#define SW_DEFAULT_MAX_CORO_NUM          3000
-#define SW_DEFAULT_STACK_SIZE            8192
-#define SW_DEFAULT_C_STACK_SIZE          (1024 * 1024 * 2)
+#define SW_DEFAULT_C_STACK_SIZE          (2 *1024 * 1024)
 #define SW_MAX_CORO_NUM_LIMIT            9223372036854775807LL
 #define SW_MAX_CORO_NESTING_LEVEL        128
 
+#define SW_CORO_SWAP_BAILOUT
+//#define SW_CORO_ZEND_TRY
+
 #ifdef SW_DEBUG
 #ifndef SW_LOG_TRACE_OPEN
 #define SW_LOG_TRACE_OPEN
