@@ -27,13 +27,15 @@
 #include <sys/file.h>
 #include <sys/statvfs.h>
 
+#include <string>
 #include <unordered_map>
 
 using namespace swoole;
+using namespace std;
 
 typedef struct
 {
-    php_context context;
+    php_coro_context context;
     int fd;
     zend_string *buf;
     uint32_t nbytes;
@@ -42,7 +44,7 @@ typedef struct
 
 typedef struct
 {
-    std::unordered_map<long, Coroutine*>::iterator _cursor;
+    unordered_map<long, Coroutine*>::iterator _cursor;
     int index;
 } coroutine_iterator;
 
@@ -88,6 +90,7 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_gethostbyname, 0, 0, 1)
     ZEND_ARG_INFO(0, domain_name)
     ZEND_ARG_INFO(0, family)
+    ZEND_ARG_INFO(0, timeout)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_defer, 0, 0, 1)
@@ -149,7 +152,7 @@ static PHP_METHOD(swoole_coroutine_iterator, __destruct);
 static PHP_METHOD(swoole_exit_exception, getFlags);
 static PHP_METHOD(swoole_exit_exception, getStatus);
 
-static std::unordered_map<int, Coroutine *> user_yield_coros;
+static unordered_map<int, Coroutine *> user_yield_coros;
 
 static zend_class_entry swoole_coroutine_util_ce;
 static zend_class_entry *swoole_coroutine_util_ce_ptr;
@@ -225,7 +228,7 @@ static int coro_exit_handler(zend_execute_data *execute_data)
     zval ex;
     zend_object *obj;
     zend_long flags = 0;
-    if (sw_coro_is_in())
+    if (PHPCoroutine::is_in())
     {
         flags |= SW_EXIT_IN_COROUTINE;
     }
@@ -233,7 +236,7 @@ static int coro_exit_handler(zend_execute_data *execute_data)
     {
         flags |= SW_EXIT_IN_SERVER;
     }
-    if (flags == SW_EXIT_IN_COROUTINE && swCoroG.count() == 1)
+    if (flags == SW_EXIT_IN_COROUTINE && Coroutine::count() == 1)
     {
         php_swoole_event_exit();
     }
@@ -282,7 +285,7 @@ static int coro_exit_handler(zend_execute_data *execute_data)
 
 void swoole_coroutine_util_init(int module_number)
 {
-    coro_init();
+    PHPCoroutine::init();
 
     SWOOLE_INIT_CLASS_ENTRY(swoole_coroutine_util, "Swoole\\Coroutine", "swoole_coroutine", "Co", swoole_coroutine_util_methods);
     SWOOLE_SET_CLASS_SERIALIZABLE(swoole_coroutine_util, zend_class_serialize_deny, zend_class_unserialize_deny);
@@ -326,7 +329,7 @@ static PHP_METHOD(swoole_exit_exception, getFlags)
 
 static PHP_METHOD(swoole_exit_exception, getStatus)
 {
-    RETURN_ZVAL(sw_zend_read_property(Z_OBJCE_P(getThis()), getThis(), ZEND_STRL("status"), 1), 0, 0);
+    RETURN_ZVAL(sw_zend_read_property(Z_OBJCE_P(getThis()), getThis(), ZEND_STRL("status"), 1), 1, 0);
 }
 
 /*
@@ -334,7 +337,7 @@ static PHP_METHOD(swoole_exit_exception, getStatus)
  */
 static PHP_METHOD(swoole_coroutine_util, yield)
 {
-    Coroutine* co = coroutine_get_current();
+    Coroutine* co = Coroutine::get_current();
     if (unlikely(!co))
     {
         swoole_php_fatal_error(E_ERROR, "can not yield outside coroutine");
@@ -351,34 +354,32 @@ static PHP_METHOD(swoole_coroutine_util, set)
     HashTable *vht = NULL;
     zval *v;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &zset) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+        Z_PARAM_ARRAY(zset)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
     php_swoole_array_separate(zset);
     vht = Z_ARRVAL_P(zset);
     if (php_swoole_array_get_value(vht, "max_coroutine", v))
     {
+        zend_long max_num;
         convert_to_long(v);
-        COROG.max_coro_num = (uint64_t) Z_LVAL_P(v);
-        if (COROG.max_coro_num <= 0)
-        {
-            COROG.max_coro_num = SW_DEFAULT_MAX_CORO_NUM;
-        }
+        max_num = Z_LVAL_P(v);
+        PHPCoroutine::set_max_num(max_num <= 0 ? SW_DEFAULT_MAX_CORO_NUM : max_num);
     }
-    if (php_swoole_array_get_value(vht, "stack_size", v))
+    if (php_swoole_array_get_value(vht, "c_stack_size", v) || php_swoole_array_get_value(vht, "stack_size", v))
     {
+        zend_long c_stack_size;
         convert_to_long(v);
-        COROG.stack_size = (uint32_t) Z_LVAL_P(v);
-        sw_coro_set_stack_size(COROG.stack_size);
+        c_stack_size = Z_LVAL_P(v);
+        Coroutine::set_stack_size(c_stack_size);
     }
     if (php_swoole_array_get_value(vht, "socket_connect_timeout", v))
     {
         convert_to_double(v);
         if (Z_DVAL_P(v))
         {
-            COROG.socket_connect_timeout = (double) Z_DVAL_P(v);
+            PHPCoroutine::socket_connect_timeout = (double) Z_DVAL_P(v);
         }
     }
     if (php_swoole_array_get_value(vht, "socket_timeout", v))
@@ -386,7 +387,7 @@ static PHP_METHOD(swoole_coroutine_util, set)
         convert_to_double(v);
         if (Z_DVAL_P(v))
         {
-            COROG.socket_timeout = (double) Z_DVAL_P(v);
+            PHPCoroutine::socket_timeout = (double) Z_DVAL_P(v);
         }
     }
     if (php_swoole_array_get_value(vht, "log_level", v))
@@ -401,9 +402,24 @@ static PHP_METHOD(swoole_coroutine_util, set)
         convert_to_long(v);
         SwooleG.trace_flags = (uint32_t) MAX(0, Z_LVAL_P(v));
     }
+    if (php_swoole_array_get_value(vht, "dns_cache_expire", v))
+    {
+        convert_to_long(v);
+        set_dns_cache_expire((time_t) Z_LVAL_P(v));
+    }
+    if (php_swoole_array_get_value(vht, "dns_cache_capacity", v))
+    {
+        convert_to_long(v);
+        set_dns_cache_capacity((size_t) Z_LVAL_P(v));
+    }
     zval_ptr_dtor(zset);
 }
 
+PHP_FUNCTION(swoole_clear_dns_cache)
+{
+    clear_dns_cache();
+}
+
 PHP_FUNCTION(swoole_coroutine_create)
 {
     zend_fcall_info fci = empty_fcall_info;
@@ -424,7 +440,7 @@ PHP_FUNCTION(swoole_coroutine_create)
         }
     }
 
-    long cid = sw_coro_create(&fci_cache, fci.param_count, fci.params);
+    long cid = PHPCoroutine::create(&fci_cache, fci.param_count, fci.params);
     if (likely(cid > 0))
     {
         RETURN_LONG(cid);
@@ -459,24 +475,24 @@ static PHP_METHOD(swoole_coroutine_util, resume)
 static PHP_METHOD(swoole_coroutine_util, stats)
 {
     array_init(return_value);
-    add_assoc_long_ex(return_value, ZEND_STRL("stack_size"), COROG.stack_size);
-    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_num"), swCoroG.count());
-    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_peak_num"), COROG.peak_coro_num);
+    add_assoc_long_ex(return_value, ZEND_STRL("c_stack_size"), Coroutine::get_stack_size());
+    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_num"), Coroutine::count());
+    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_peak_num"), Coroutine::get_peak_num());
 }
 
 static PHP_METHOD(swoole_coroutine_util, getCid)
 {
-    RETURN_LONG(sw_get_current_cid());
+    RETURN_LONG(PHPCoroutine::get_cid());
 }
 
 int php_coroutine_reactor_can_exit(swReactor *reactor)
 {
-    return swCoroG.count() == 0;
+    return Coroutine::count() == 0;
 }
 
 static PHP_METHOD(swoole_coroutine_util, sleep)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     double seconds;
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "d", & seconds) == FAILURE)
@@ -518,9 +534,9 @@ static void aio_onReadCompleted(swAio_event *event)
         ZVAL_BOOL(result, 0);
     }
 
-    php_context *context = (php_context *) event->object;
-    int ret = sw_coro_resume(context, result, retval);
-    if (ret == CORO_END && retval)
+    php_coro_context *context = (php_coro_context *) event->object;
+    int ret = PHPCoroutine::resume_m(context, result, retval);
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -545,7 +561,7 @@ static void aio_onFgetsCompleted(swAio_event *event)
         ZVAL_BOOL(result, 0);
     }
 
-    php_context *context = (php_context *) event->object;
+    php_coro_context *context = (php_coro_context *) event->object;
     php_stream *stream;
     php_stream_from_zval_no_verify(stream, &context->coro_params);
 
@@ -554,8 +570,8 @@ static void aio_onFgetsCompleted(swAio_event *event)
         stream->eof = 1;
     }
 
-    int ret = sw_coro_resume(context, result, retval);
-    if (ret == CORO_END && retval)
+    int ret = PHPCoroutine::resume_m(context, result, retval);
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -579,9 +595,9 @@ static void aio_onWriteCompleted(swAio_event *event)
         ZVAL_LONG(result, event->ret);
     }
 
-    php_context *context = (php_context *) event->object;
-    int ret = sw_coro_resume(context, result, retval);
-    if (ret == CORO_END && retval)
+    php_coro_context *context = (php_coro_context *) event->object;
+    int ret = PHPCoroutine::resume_m(context, result, retval);
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -593,7 +609,7 @@ static void aio_onWriteCompleted(swAio_event *event)
 static int co_socket_onReadable(swReactor *reactor, swEvent *event)
 {
     util_socket *sock = (util_socket *) event->socket->object;
-    php_context *context = &sock->context;
+    php_coro_context *context = &sock->context;
 
     zval *retval = NULL;
     zval result;
@@ -623,9 +639,9 @@ static int co_socket_onReadable(swReactor *reactor, swEvent *event)
         sock->buf->len = n;
         ZVAL_STR(&result, sock->buf);
     }
-    int ret = sw_coro_resume(context, &result, retval);
+    int ret = PHPCoroutine::resume_m(context, &result, retval);
     zval_ptr_dtor(&result);
-    if (ret == CORO_END && retval)
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -636,7 +652,7 @@ static int co_socket_onReadable(swReactor *reactor, swEvent *event)
 static int co_socket_onWritable(swReactor *reactor, swEvent *event)
 {
     util_socket *sock = (util_socket *) event->socket->object;
-    php_context *context = &sock->context;
+    php_coro_context *context = &sock->context;
 
     zval *retval = NULL;
     zval result;
@@ -659,9 +675,9 @@ static int co_socket_onWritable(swReactor *reactor, swEvent *event)
     {
         ZVAL_LONG(&result, n);
     }
-    int ret = sw_coro_resume(context, &result, retval);
+    int ret = PHPCoroutine::resume_m(context, &result, retval);
     zval_ptr_dtor(&result);
-    if (ret == CORO_END && retval)
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -695,8 +711,7 @@ static void co_socket_read(int fd, zend_long length, INTERNAL_FUNCTION_PARAMETER
 
     sock->context.state = SW_CORO_CONTEXT_RUNNING;
 
-    sw_coro_save(return_value, &sock->context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, &sock->context);
 }
 
 static void co_socket_write(int fd, char* str, size_t l_str, INTERNAL_FUNCTION_PARAMETERS)
@@ -727,19 +742,18 @@ static void co_socket_write(int fd, char* str, size_t l_str, INTERNAL_FUNCTION_P
     bzero(sock, sizeof(util_socket));
     _socket->object = sock;
 
-    php_context *context = &sock->context;
+    php_coro_context *context = &sock->context;
     context->state = SW_CORO_CONTEXT_RUNNING;
     context->private_data = str;
 
     sock->nbytes = l_str;
 
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_coroutine_util, fread)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     zval *handle;
     zend_long length = 0;
@@ -798,7 +812,7 @@ static PHP_METHOD(swoole_coroutine_util, fread)
         RETURN_FALSE;
     }
 
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
+    php_coro_context *context = (php_coro_context *) emalloc(sizeof(php_coro_context));
 
     ((char *) ev.buf)[length] = 0;
     ev.flags = 0;
@@ -809,8 +823,6 @@ static PHP_METHOD(swoole_coroutine_util, fread)
     ev.fd = fd;
     ev.offset = _seek;
 
-    php_swoole_check_aio();
-
     swTrace("fd=%d, offset=%jd, length=%ld", fd, (intmax_t) ev.offset, ev.nbytes);
 
     int ret = swAio_dispatch(&ev);
@@ -822,13 +834,12 @@ static PHP_METHOD(swoole_coroutine_util, fread)
 
     context->state = SW_CORO_CONTEXT_RUNNING;
 
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_coroutine_util, fgets)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     zval *handle;
     php_stream *stream;
@@ -882,7 +893,7 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
         RETURN_FALSE;
     }
 
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
+    php_coro_context *context = (php_coro_context *) emalloc(sizeof(php_coro_context));
 
     ev.flags = 0;
     ev.type = SW_AIO_FGETS;
@@ -892,8 +903,6 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
     ev.fd = fd;
     ev.req = (void *) file;
 
-    php_swoole_check_aio();
-
     swTrace("fd=%d, offset=%jd, length=%ld", fd, (intmax_t) ev.offset, ev.nbytes);
 
     int ret = swAio_dispatch(&ev);
@@ -906,13 +915,12 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
     context->coro_params = *handle;
     context->state = SW_CORO_CONTEXT_RUNNING;
 
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_coroutine_util, fwrite)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     zval *handle;
     char *str;
@@ -961,7 +969,7 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
         RETURN_FALSE;
     }
 
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
+    php_coro_context *context = (php_coro_context *) emalloc(sizeof(php_coro_context));
 
     ev.flags = 0;
     ev.type = SW_AIO_WRITE;
@@ -971,8 +979,6 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
     ev.fd = fd;
     ev.offset = _seek;
 
-    php_swoole_check_aio();
-
     swTrace("fd=%d, offset=%jd, length=%ld", fd, (intmax_t) ev.offset, ev.nbytes);
 
     int ret = swAio_dispatch(&ev);
@@ -984,26 +990,23 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
 
     context->state = SW_CORO_CONTEXT_RUNNING;
 
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_coroutine_util, readFile)
 {
-    coro_check();
+    PHPCoroutine::check();
 
-    char *filename = NULL;
-    size_t l_filename = 0;
+    char *filename;
+    size_t l_filename;
     zend_long flags = 0;
 
-    ZEND_PARSE_PARAMETERS_START(1, 1)
+    ZEND_PARSE_PARAMETERS_START(1, 2)
         Z_PARAM_STRING(filename, l_filename)
         Z_PARAM_OPTIONAL
         Z_PARAM_LONG(flags)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    php_swoole_check_aio();
-
     swString *result = Coroutine::read_file(filename, flags & LOCK_EX);
     if (result == NULL)
     {
@@ -1018,12 +1021,12 @@ static PHP_METHOD(swoole_coroutine_util, readFile)
 
 static PHP_METHOD(swoole_coroutine_util, writeFile)
 {
-    coro_check();
+    PHPCoroutine::check();
 
-    char *filename = NULL;
-    size_t l_filename = 0;
-    char *data = NULL;
-    size_t l_data = 0;
+    char *filename;
+    size_t l_filename;
+    char *data;
+    size_t l_data;
     zend_long flags = 0;
 
     ZEND_PARSE_PARAMETERS_START(2, 3)
@@ -1056,7 +1059,7 @@ static PHP_METHOD(swoole_coroutine_util, writeFile)
 
 static void coro_dns_onResolveCompleted(swAio_event *event)
 {
-    php_context *context = (php_context *) event->object;
+    php_coro_context *context = (php_coro_context *) event->object;
 
     zval *retval = NULL;
     zval *result = NULL;
@@ -1073,8 +1076,8 @@ static void coro_dns_onResolveCompleted(swAio_event *event)
         ZVAL_BOOL(result, 0);
     }
 
-    int ret = sw_coro_resume(context, result, retval);
-    if (ret == CORO_END && retval)
+    int ret = PHPCoroutine::resume_m(context, result, retval);
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -1085,7 +1088,7 @@ static void coro_dns_onResolveCompleted(swAio_event *event)
 
 static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
 {
-    php_context *context = (php_context *) event->object;
+    php_coro_context *context = (php_coro_context *) event->object;
 
     zval *retval = NULL;
     zval *result = NULL;
@@ -1128,8 +1131,8 @@ static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
         SwooleG.error = req->error;
     }
 
-    int ret = sw_coro_resume(context, result, retval);
-    if (ret == CORO_END && retval)
+    int ret = PHPCoroutine::resume_m(context, result, retval);
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -1146,18 +1149,19 @@ static void coro_dns_onGetaddrinfoCompleted(swAio_event *event)
 
 PHP_FUNCTION(swoole_coroutine_gethostbyname)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     char *domain_name;
     size_t l_domain_name;
-    long family = AF_INET;
+    zend_long family = AF_INET;
+    double timeout = -1;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|l", &domain_name, &l_domain_name, &family) == FAILURE)
+    if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|ld", &domain_name, &l_domain_name, &family, &timeout) == FAILURE)
     {
         RETURN_FALSE;
     }
 
-    if (l_domain_name <= 0)
+    if (l_domain_name == 0)
     {
         swoole_php_fatal_error(E_WARNING, "domain name is empty.");
         RETURN_FALSE;
@@ -1169,66 +1173,36 @@ PHP_FUNCTION(swoole_coroutine_gethostbyname)
         RETURN_FALSE;
     }
 
-    swAio_event ev;
-    bzero(&ev, sizeof(swAio_event));
-
-    if (l_domain_name < SW_IP_MAX_LENGTH)
+    string address = Coroutine::gethostbyname(string(domain_name, l_domain_name), family, timeout);
+    if (address.empty())
     {
-        ev.nbytes = SW_IP_MAX_LENGTH;
+        RETURN_FALSE;
     }
     else
     {
-        ev.nbytes = l_domain_name + 1;
+        RETURN_STRINGL(address.c_str(), address.length());
     }
-
-    ev.buf = emalloc(ev.nbytes);
-    if (!ev.buf)
-    {
-        swWarn("malloc failed.");
-        RETURN_FALSE;
-    }
-
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
-
-    memcpy(ev.buf, domain_name, l_domain_name);
-    ((char *) ev.buf)[l_domain_name] = 0;
-    ev.flags = family;
-    ev.type = SW_AIO_GETHOSTBYNAME;
-    ev.object = context;
-    ev.handler = swAio_handler_gethostbyname;
-    ev.callback = coro_dns_onResolveCompleted;
-
-    php_swoole_check_aio();
-
-    if (swAio_dispatch(&ev) < 0)
-    {
-        efree(ev.buf);
-        RETURN_FALSE;
-    }
-
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
 }
 
 static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     char *hostname;
     size_t l_hostname;
-    long family = AF_INET;
-    long socktype = SOCK_STREAM;
-    long protocol = IPPROTO_TCP;
+    zend_long family = AF_INET;
+    zend_long socktype = SOCK_STREAM;
+    zend_long protocol = IPPROTO_TCP;
     char *service = NULL;
     size_t l_service = 0;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|llls", &hostname, &l_hostname, &family, socktype, &protocol,
-            &hostname, &l_hostname) == FAILURE)
+    if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|llls", &hostname, &l_hostname, &family, &socktype, &protocol,
+            &service, &l_service) == FAILURE)
     {
         RETURN_FALSE;
     }
 
-    if (l_hostname <= 0)
+    if (l_hostname == 0)
     {
         swoole_php_fatal_error(E_WARNING, "hostname is empty.");
         RETURN_FALSE;
@@ -1246,7 +1220,7 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
     swRequest_getaddrinfo *req = (swRequest_getaddrinfo *) emalloc(sizeof(swRequest_getaddrinfo));
     bzero(req, sizeof(swRequest_getaddrinfo));
 
-    php_context *context = (php_context *) emalloc(sizeof(php_context));
+    php_coro_context *context = (php_coro_context *) emalloc(sizeof(php_coro_context));
 
     ev.type = SW_AIO_GETADDRINFO;
     ev.object = context;
@@ -1259,7 +1233,7 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
     req->socktype = socktype;
     req->protocol = protocol;
 
-    if (service)
+    if (l_service > 0)
     {
         req->service = estrndup(service, l_service);
     }
@@ -1273,16 +1247,13 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
         req->result = ecalloc(SW_DNS_HOST_BUFFER_SIZE, sizeof(struct sockaddr_in6));
     }
 
-    php_swoole_check_aio();
-
     if (swAio_dispatch(&ev) < 0)
     {
         efree(ev.buf);
         RETURN_FALSE;
     }
 
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_coroutine_util, getBackTrace)
@@ -1291,17 +1262,17 @@ static PHP_METHOD(swoole_coroutine_util, getBackTrace)
     zend_long options = DEBUG_BACKTRACE_PROVIDE_OBJECT;
     zend_long limit = 0;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|ll", &cid) == FAILURE)
+    if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|ll", &cid, &options, &limit) == FAILURE)
     {
         RETURN_FALSE;
     }
-    if (cid == sw_get_current_cid())
+    if (cid == PHPCoroutine::get_cid())
     {
         zend_fetch_debug_backtrace(return_value, 0, options, limit);
     }
     else
     {
-        coro_task *task = (coro_task *) coroutine_get_task_by_cid(cid);
+        php_coro_task *task = (php_coro_task *) Coroutine::get_task_by_cid(cid);
         if (task == NULL)
         {
             RETURN_FALSE;
@@ -1316,14 +1287,14 @@ static PHP_METHOD(swoole_coroutine_util, getBackTrace)
 static PHP_METHOD(swoole_coroutine_iterator, rewind)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    itearator->_cursor = swCoroG.coroutines.begin();
+    itearator->_cursor = Coroutine::coroutines.begin();
     itearator->index = 0;
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, valid)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    RETURN_BOOL(itearator->_cursor != swCoroG.coroutines.end());
+    RETURN_BOOL(itearator->_cursor != Coroutine::coroutines.end());
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, current)
@@ -1348,7 +1319,7 @@ PHP_METHOD(swoole_coroutine_iterator, key)
 
 static PHP_METHOD(swoole_coroutine_iterator, count)
 {
-    RETURN_LONG(swCoroG.count());
+    RETURN_LONG(Coroutine::count());
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, __destruct)
@@ -1368,7 +1339,7 @@ static PHP_METHOD(swoole_coroutine_util, listCoroutines)
 
 static PHP_METHOD(swoole_coroutine_util, statvfs)
 {
-    coro_check();
+    PHPCoroutine::check();
 
     char *path;
     size_t l_path;
@@ -1411,7 +1382,7 @@ PHP_FUNCTION(swoole_coroutine_exec)
         RETURN_FALSE;
     }
 
-    coro_check();
+    PHPCoroutine::check();
     swoole_coroutine_signal_init();
     php_swoole_check_reactor();
 
@@ -1502,7 +1473,7 @@ PHP_FUNCTION(swoole_coroutine_defer)
     zend_fcall_info_cache fci_cache = empty_fcall_info_cache;
     php_defer_fci *defer_fci;
 
-    coro_check();
+    PHPCoroutine::check();
 
     ZEND_PARSE_PARAMETERS_START(1, -1)
         Z_PARAM_FUNC(fci, fci_cache)
@@ -1513,5 +1484,5 @@ PHP_FUNCTION(swoole_coroutine_defer)
     defer_fci->fci = fci;
     defer_fci->fci_cache = fci_cache;
     sw_fci_cache_persist(&defer_fci->fci_cache);
-    sw_coro_add_defer_task(coro_onDefer, defer_fci);
+    PHPCoroutine::defer(coro_onDefer, defer_fci);
 }
