@@ -18,17 +18,14 @@
 
 #include "php_swoole.h"
 
-#ifdef SW_COROUTINE
 #include "swoole_http_client.h"
 #include "swoole_coroutine.h"
 #include "coroutine_c_api.h"
-#include "socket.h"
 
 using namespace swoole;
 
 extern swString *http_client_buffer;
 
-extern void php_swoole_client_coro_check_setting(Socket *cli, zval *zset);
 extern bool php_swoole_client_coro_socket_free(Socket *cli);
 
 static int http_parser_on_header_field(swoole_http_parser *parser, const char *at, size_t length);
@@ -51,88 +48,84 @@ static const swoole_http_parser_settings http_parser_settings =
     http_parser_on_message_complete
 };
 
-namespace swoole
+class http_client
 {
-    class PHPHttpClient
-    {
-        private:
-        Socket* socket = nullptr;
-        std::string addr = "";
-        enum swSocket_type sock_type = SW_SOCK_TCP;
+    private:
+    Socket* socket = nullptr;
+    swSocket_type socket_type = SW_SOCK_TCP;
 
-        public:
-        /* states */
-        http_client_state state = HTTP_CLIENT_STATE_WAIT;
-        bool wait = false;
-        bool defer = false;
+    public:
+    /* states */
+    http_client_state state = HTTP_CLIENT_STATE_WAIT;
+    bool wait = false;
+    bool defer = false;
 
-        /* request info */
-        std::string host = "";
-        uint16_t port = 80;
-    #ifdef SW_USE_OPENSSL
-        uint8_t ssl = false;
-    #endif
-        double connect_timeout = COROG.socket_connect_timeout;
-        double timeout = COROG.socket_timeout;
-        int8_t method = SW_HTTP_GET;       // method
-        std::string uri;
-
-        /* response parse */
-        swoole_http_parser parser = {0};
-        char *tmp_header_field_name = nullptr;
-        int tmp_header_field_name_len = 0;
-        swString *body = nullptr;
-    #ifdef SW_HAVE_ZLIB
-        z_stream gzip_stream = {0};
-        swString *gzip_buffer = nullptr;
-        swString *_gzip_buffer = nullptr;
-    #endif
-
-        /* options */
-        uint8_t reconnect_interval = 1;
-        uint8_t reconnected_count = 0;
-        bool keep_alive = true;          // enable default
-        bool websocket = false;            // if upgrade successfully
-        bool gzip = false;               // enable gzip
-        bool chunked = false;            // Transfer-Encoding: chunked
-        bool completed = false;          // response parse over
-        bool websocket_mask = false;     // enable websocket mask
-        bool is_download = false;        // save http response to file
-        int download_file_fd = 0;
-        bool has_upload_files = false;
-
-        /* safety zval */
-        zval _zobject;
-        zval *zobject = &_zobject;
-
-        PHPHttpClient(zval* zobject, std::string host, zend_long port = 80, zend_bool ssl = false);
-
-        private:
-#ifdef SW_HAVE_ZLIB
-        void init_gzip();
+    /* request info */
+    std::string host = "127.0.0.1";
+    uint16_t port = 80;
+#ifdef SW_USE_OPENSSL
+    uint8_t ssl = false;
 #endif
-        bool connect();
-        bool keep_liveness();
-        bool send();
-        void reset();
+    double connect_timeout = PHPCoroutine::socket_connect_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
+    int8_t method = SW_HTTP_GET;       // method
+    std::string uri;
 
-        public:
+    /* response parse */
+    swoole_http_parser parser = {0};
+    char *tmp_header_field_name = nullptr;
+    int tmp_header_field_name_len = 0;
+    swString *body = nullptr;
 #ifdef SW_HAVE_ZLIB
-        bool init_compression(http_compress_method method);
-        bool uncompress_response();
+    z_stream gzip_stream = {0};
+    swString *gzip_buffer = nullptr;
+    swString *_gzip_buffer = nullptr;
 #endif
-        void check_bind();
-        void set(zval *zset);
-        bool exec(std::string uri);
-        bool recv(double timeout = 0);
-        void recv(zval *zframe, double timeout = 0);
-        bool upgrade(std::string uri);
-        bool push(zval *zdata, zend_long opcode = WEBSOCKET_OPCODE_TEXT, bool _fin = true);
-        bool close();
 
-        ~PHPHttpClient();
-    };
-}
+    /* options */
+    uint8_t reconnect_interval = 1;
+    uint8_t reconnected_count = 0;
+    bool keep_alive = true;          // enable default
+    bool websocket = false;            // if upgrade successfully
+    bool gzip = false;               // enable gzip
+    bool chunked = false;            // Transfer-Encoding: chunked
+    bool completed = false;          // response parse over
+    bool websocket_mask = false;     // enable websocket mask
+    bool is_download = false;        // save http response to file
+    int download_file_fd = 0;
+    bool has_upload_files = false;
+
+    /* safety zval */
+    zval _zobject;
+    zval *zobject = &_zobject;
+
+    http_client(zval* zobject, std::string host, zend_long port = 80, zend_bool ssl = false);
+
+    private:
+#ifdef SW_HAVE_ZLIB
+    void init_gzip();
+#endif
+    bool connect();
+    bool keep_liveness();
+    bool send();
+    void reset();
+
+    public:
+#ifdef SW_HAVE_ZLIB
+    bool init_compression(http_compress_method method);
+    bool uncompress_response();
+#endif
+    void check_bind();
+    void set(zval *zset);
+    bool exec(std::string uri);
+    bool recv(double timeout = 0);
+    void recv(zval *zframe, double timeout = 0);
+    bool upgrade(std::string uri);
+    bool push(zval *zdata, zend_long opcode = WEBSOCKET_OPCODE_TEXT, bool _fin = true);
+    bool close();
+
+    ~http_client();
+};
 
 static zend_class_entry swoole_http_client_coro_ce;
 zend_class_entry *swoole_http_client_coro_ce_ptr;
@@ -140,7 +133,7 @@ static zend_object_handlers swoole_http_client_coro_handlers;
 
 typedef struct
 {
-    PHPHttpClient* phc;
+    http_client* phc;
     zend_object std;
 } http_client_coro;
 
@@ -272,7 +265,7 @@ static const zend_function_entry swoole_http_client_coro_methods[] =
 
 static int http_parser_on_header_field(swoole_http_parser *parser, const char *at, size_t length)
 {
-    PHPHttpClient* http = (PHPHttpClient*) parser->data;
+    http_client* http = (http_client*) parser->data;
     http->tmp_header_field_name = (char *) at;
     http->tmp_header_field_name_len = length;
     return 0;
@@ -280,7 +273,7 @@ static int http_parser_on_header_field(swoole_http_parser *parser, const char *a
 
 static int http_parser_on_header_value(swoole_http_parser *parser, const char *at, size_t length)
 {
-    PHPHttpClient* http = (PHPHttpClient*) parser->data;
+    http_client* http = (http_client*) parser->data;
     zval* zobject = (zval*) http->zobject;
     zval *zheaders = sw_zend_read_property_array(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("headers"), 1);
     char *header_name = zend_str_tolower_dup(http->tmp_header_field_name, http->tmp_header_field_name_len);
@@ -319,7 +312,7 @@ static int http_parser_on_header_value(swoole_http_parser *parser, const char *a
 
 static int http_parser_on_headers_complete(swoole_http_parser *parser)
 {
-    PHPHttpClient* http = (PHPHttpClient*) parser->data;
+    http_client* http = (http_client*) parser->data;
     //no content-length
     if (http->chunked == 0 && parser->content_length == -1)
     {
@@ -336,7 +329,7 @@ static int http_parser_on_headers_complete(swoole_http_parser *parser)
 
 static int http_parser_on_body(swoole_http_parser *parser, const char *at, size_t length)
 {
-    PHPHttpClient* http = (PHPHttpClient*) parser->data;
+    http_client* http = (http_client*) parser->data;
     if (swString_append_ptr(http->body, at, length) < 0)
     {
         return -1;
@@ -370,7 +363,7 @@ static int http_parser_on_body(swoole_http_parser *parser, const char *at, size_
 
 static int http_parser_on_message_complete(swoole_http_parser *parser)
 {
-    PHPHttpClient* http = (PHPHttpClient*) parser->data;
+    http_client* http = (http_client*) parser->data;
     zval* zobject = (zval*) http->zobject;
 
     if (parser->upgrade && !http->websocket)
@@ -411,7 +404,7 @@ static int http_parser_on_message_complete(swoole_http_parser *parser)
     }
 }
 
-PHPHttpClient::PHPHttpClient(zval* zobject, std::string host, zend_long port, zend_bool ssl)
+http_client::http_client(zval* zobject, std::string host, zend_long port, zend_bool ssl)
 {
     // check host
     if (host.length() == 0)
@@ -419,13 +412,21 @@ PHPHttpClient::PHPHttpClient(zval* zobject, std::string host, zend_long port, ze
         swoole_php_fatal_error(E_ERROR, "host is empty.");
         return;
     }
-    this->host = this->addr = host;
-
-    // parse addr and sock_type
-    sock_type = get_socket_type_from_uri(addr, true);
-
-    // checo port
-    if (sock_type == SW_SOCK_TCP || sock_type == SW_SOCK_TCP6)
+    if (host.compare(0, 6, "unix:/", 0, 6) == 0)
+    {
+        host = host.substr(sizeof("unix:") - 1);
+        host.erase(0, host.find_first_not_of('/') - 1);
+        socket_type = SW_SOCK_UNIX_STREAM;
+    }
+    else if (host.find(':') != std::string::npos)
+    {
+        socket_type = SW_SOCK_TCP6;
+    }
+    else
+    {
+        socket_type = SW_SOCK_TCP;
+    }
+    if (socket_type == SW_SOCK_TCP || socket_type == SW_SOCK_TCP6)
     {
         if (port <= 0 || port > SW_CLIENT_MAX_PORT)
         {
@@ -433,9 +434,8 @@ PHPHttpClient::PHPHttpClient(zval* zobject, std::string host, zend_long port, ze
             return;
         }
     }
+    this->host = host;
     this->port = port;
-
-    // check ssl
 #ifdef SW_USE_OPENSSL
     this->ssl = ssl;
 #else
@@ -455,7 +455,7 @@ PHPHttpClient::PHPHttpClient(zval* zobject, std::string host, zend_long port, ze
 }
 
 #ifdef SW_HAVE_ZLIB
-void PHPHttpClient::init_gzip()
+void http_client::init_gzip()
 {
     gzip = true;
     memset(&gzip_stream, 0, sizeof(gzip_stream));
@@ -475,7 +475,7 @@ void PHPHttpClient::init_gzip()
     gzip_stream.zfree = php_zlib_free;
 }
 
-bool PHPHttpClient::init_compression(http_compress_method method)
+bool http_client::init_compression(http_compress_method method)
 {
     switch(method)
     {
@@ -502,7 +502,7 @@ bool PHPHttpClient::init_compression(http_compress_method method)
     return true;
 }
 
-bool PHPHttpClient::uncompress_response()
+bool http_client::uncompress_response()
 {
     int status = 0;
 
@@ -549,15 +549,15 @@ bool PHPHttpClient::uncompress_response()
 }
 #endif
 
-void PHPHttpClient::check_bind()
+void http_client::check_bind()
 {
     if (socket)
     {
-        sw_coro_check_bind("http client", socket->has_bound());
+        PHPCoroutine::check_bind("http client", socket->has_bound());
     }
 }
 
-void PHPHttpClient::set(zval *zset = nullptr)
+void http_client::set(zval *zset = nullptr)
 {
     zval *ztmp;
     HashTable *vht;
@@ -606,44 +606,24 @@ void PHPHttpClient::set(zval *zset = nullptr)
             websocket_mask = Z_BVAL_P(ztmp);
         }
     }
-    else
+    if (socket)
     {
-        SW_ASSERT(socket);
-        // will be set after create socket and before connect
-        php_swoole_client_coro_check_setting(socket, zsettings);
-        if (socket->http_proxy)
-        {
-            if (socket->http_proxy->password)
-            {
-                char _buf1[256];
-                int _n1 = snprintf(
-                    _buf1, sizeof(_buf1), "%*s:%*s",
-                    socket->http_proxy->l_user, socket->http_proxy->user,
-                    socket->http_proxy->l_password, socket->http_proxy->password
-                );
-                zend_string *str = php_base64_encode((const unsigned char *) _buf1, _n1);
-                snprintf(
-                    socket->http_proxy->buf, sizeof(socket->http_proxy->buf),
-                    "Proxy-Authorization:Basic %*s", (int)str->len, str->val
-                );
-                zend_string_free(str);
-            }
-        }
+        sw_coro_socket_set(socket, zset ? zset : zsettings);
     }
 }
 
-bool PHPHttpClient::connect()
+bool http_client::connect()
 {
     if (!socket)
     {
-        socket = new Socket(sock_type);
+        socket = new Socket(socket_type);
         if (unlikely(socket->socket == nullptr))
         {
-            delete socket;
-            socket = nullptr;
             swoole_php_fatal_error(E_WARNING, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
             zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), errno);
             zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), strerror(errno));
+            delete socket;
+            socket = nullptr;
             return false;
         }
 #ifdef SW_USE_OPENSSL
@@ -653,8 +633,8 @@ bool PHPHttpClient::connect()
         set();
 
         // connect
-        socket->set_timeout(connect_timeout, true);
-        if (!socket->connect(addr, port))
+        socket->set_timeout(connect_timeout);
+        if (!socket->connect(host, port))
         {
             zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
             zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), socket->errMsg);
@@ -682,7 +662,7 @@ bool PHPHttpClient::connect()
     return true;
 }
 
-bool PHPHttpClient::keep_liveness()
+bool http_client::keep_liveness()
 {
     if (!socket || !socket->check_liveness())
     {
@@ -705,7 +685,7 @@ bool PHPHttpClient::keep_liveness()
     return true;
 }
 
-bool PHPHttpClient::send()
+bool http_client::send()
 {
     zval *value = NULL;
     char *method;
@@ -1191,7 +1171,7 @@ bool PHPHttpClient::send()
     swTraceLog(
         SW_TRACE_HTTP_CLIENT,
         "to [%s:%u%s] by fd#%d in cid#%ld with [%zu] bytes: <<EOF\n%.*s\nEOF",
-        host.c_str(), port, uri.c_str(), socket->get_fd(), coroutine_get_current_cid(),
+        host.c_str(), port, uri.c_str(), socket->get_fd(), Coroutine::get_current_cid(),
         http_client_buffer->length, (int) http_client_buffer->length, http_client_buffer->str
     );
 
@@ -1209,7 +1189,7 @@ bool PHPHttpClient::send()
     return true;
 }
 
-bool PHPHttpClient::exec(std::string uri)
+bool http_client::exec(std::string uri)
 {
     this->uri = uri;
     // bzero when make a new reqeust
@@ -1224,7 +1204,7 @@ bool PHPHttpClient::exec(std::string uri)
     }
 }
 
-bool PHPHttpClient::recv(double timeout)
+bool http_client::recv(double timeout)
 {
     long parsed_n = 0;
     ssize_t total_bytes = 0, retval = 0;
@@ -1245,9 +1225,9 @@ bool PHPHttpClient::recv(double timeout)
     }
 
     buffer = socket->get_read_buffer();
+    socket->set_timer(Socket::TIMER_LV_GLOBAL, timeout);
     while (completed == 0)
     {
-        socket->set_timeout(timeout, true);
         retval = socket->recv(buffer->str, buffer->size);
         if (retval > 0)
         {
@@ -1256,11 +1236,8 @@ bool PHPHttpClient::recv(double timeout)
             swTraceLog(SW_TRACE_HTTP_CLIENT, "parsed_n=%ld, retval=%ld, total_bytes=%ld, completed=%d.", parsed_n, retval, total_bytes, completed);
             if (unlikely(parser.state == s_dead && !completed))
             {
-                // parse error
-                zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), EPROTO);
-                zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), strerror(EPROTO));
-                close();
-                return SW_ERR;
+                socket->set_err(EPROTO);
+                goto _error;
             }
             if (parsed_n >= 0)
             {
@@ -1274,7 +1251,9 @@ bool PHPHttpClient::recv(double timeout)
         }
         else
         {
+            _error:
             // IO error
+            socket->del_timer(Socket::TIMER_LV_GLOBAL);
             zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
             zend_update_property_string(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errMsg"), socket->errMsg);
             if (socket->errCode == ETIMEDOUT)
@@ -1289,6 +1268,8 @@ bool PHPHttpClient::recv(double timeout)
             return false;
         }
     }
+    socket->del_timer(Socket::TIMER_LV_GLOBAL);
+
     /**
      * TODO: Sec-WebSocket-Accept check
      */
@@ -1321,7 +1302,7 @@ bool PHPHttpClient::recv(double timeout)
     return true;
 }
 
-void PHPHttpClient::recv(zval *zframe, double timeout)
+void http_client::recv(zval *zframe, double timeout)
 {
     check_bind();
 
@@ -1334,8 +1315,10 @@ void PHPHttpClient::recv(zval *zframe, double timeout)
         zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_SERVER_RESET);
         return;
     }
-    socket->set_timeout(timeout, true);
+
+    socket->set_timer(Socket::TIMER_LV_GLOBAL, timeout);
     ssize_t retval = socket->recv_packet();
+    socket->del_timer(Socket::TIMER_LV_GLOBAL);
     if (retval <= 0)
     {
         zend_update_property_long(swoole_http_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), socket->errCode);
@@ -1355,7 +1338,7 @@ void PHPHttpClient::recv(zval *zframe, double timeout)
     }
 }
 
-bool PHPHttpClient::upgrade(std::string uri)
+bool http_client::upgrade(std::string uri)
 {
     defer = false;
     if (!websocket)
@@ -1373,7 +1356,7 @@ bool PHPHttpClient::upgrade(std::string uri)
     return websocket;
 }
 
-bool PHPHttpClient::push(zval *zdata, zend_long opcode, bool fin)
+bool http_client::push(zval *zdata, zend_long opcode, bool fin)
 {
     check_bind();
 
@@ -1413,7 +1396,7 @@ bool PHPHttpClient::push(zval *zdata, zend_long opcode, bool fin)
     }
 }
 
-void PHPHttpClient::reset()
+void http_client::reset()
 {
     wait = false;
     state = HTTP_CLIENT_STATE_READY;
@@ -1440,7 +1423,7 @@ void PHPHttpClient::reset()
     }
 }
 
-bool PHPHttpClient::close()
+bool http_client::close()
 {
     bool ret;
     if (!socket)
@@ -1462,7 +1445,7 @@ bool PHPHttpClient::close()
     return ret;
 }
 
-PHPHttpClient::~PHPHttpClient()
+http_client::~http_client()
 {
     close();
     if (body)
@@ -1483,9 +1466,9 @@ static sw_inline http_client_coro* swoole_http_client_coro_fetch_object(zend_obj
     return (http_client_coro *) ((char *) obj - swoole_http_client_coro_handlers.offset);
 }
 
-static sw_inline PHPHttpClient * swoole_get_phc(zval *zobject)
+static sw_inline http_client * swoole_get_phc(zval *zobject)
 {
-    PHPHttpClient *phc = swoole_http_client_coro_fetch_object(Z_OBJ_P(zobject))->phc;
+    http_client *phc = swoole_http_client_coro_fetch_object(Z_OBJ_P(zobject))->phc;
     if (UNEXPECTED(!phc))
     {
         swoole_php_fatal_error(E_ERROR, "you must call Http Client constructor first.");
@@ -1573,7 +1556,7 @@ static PHP_METHOD(swoole_http_client_coro, __construct)
     zend_update_property_stringl(swoole_http_client_coro_ce_ptr, getThis(), ZEND_STRL("host"), host, host_len);
     zend_update_property_long(swoole_http_client_coro_ce_ptr,getThis(), ZEND_STRL("port"), port);
     zend_update_property_bool(swoole_http_client_coro_ce_ptr,getThis(), ZEND_STRL("ssl"), ssl);
-    hcc_t->phc = new PHPHttpClient(getThis(), std::string(host, host_len), port, ssl);
+    hcc_t->phc = new http_client(getThis(), std::string(host, host_len), port, ssl);
 }
 
 static PHP_METHOD(swoole_http_client_coro, __destruct) { }
@@ -1581,7 +1564,7 @@ static PHP_METHOD(swoole_http_client_coro, __destruct) { }
 static PHP_METHOD(swoole_http_client_coro, set)
 {
     zval *zset;
-    PHPHttpClient* phc = swoole_get_phc(getThis());
+    http_client* phc = swoole_get_phc(getThis());
 
     ZEND_PARSE_PARAMETERS_START(1, 1)
         Z_PARAM_ARRAY(zset)
@@ -1594,14 +1577,14 @@ static PHP_METHOD(swoole_http_client_coro, set)
 
 static PHP_METHOD(swoole_http_client_coro, getDefer)
 {
-    PHPHttpClient *phc = swoole_get_phc(getThis());
+    http_client *phc = swoole_get_phc(getThis());
 
     RETURN_BOOL(phc->defer);
 }
 
 static PHP_METHOD(swoole_http_client_coro, setDefer)
 {
-    PHPHttpClient *phc = swoole_get_phc(getThis());
+    http_client *phc = swoole_get_phc(getThis());
     zend_bool defer = 1;
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
@@ -1723,12 +1706,12 @@ static PHP_METHOD(swoole_http_client_coro, addFile)
     {
         length = file_stat.st_size - offset;
     }
-    if (type == NULL)
+    if (l_type == 0)
     {
         type = (char *) swoole_get_mime_type(path);
         l_type = strlen(type);
     }
-    if (filename == NULL)
+    if (l_filename == 0)
     {
         char *dot = strrchr(path, '/');
         if (dot == NULL)
@@ -1777,12 +1760,12 @@ static PHP_METHOD(swoole_http_client_coro, addData)
         Z_PARAM_STRING(filename, l_filename)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    if (type == NULL)
+    if (l_type == 0)
     {
         type = (char *) "application/octet-stream";
         l_type = strlen(type);
     }
-    if (filename == NULL)
+    if (l_filename == 0)
     {
         filename = name;
         l_filename = l_name;
@@ -1804,7 +1787,7 @@ static PHP_METHOD(swoole_http_client_coro, addData)
 
 static PHP_METHOD(swoole_http_client_coro, execute)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
+    http_client* phc = swoole_get_phc(getThis());
     char *uri = NULL;
     size_t uri_len = 0;
 
@@ -1817,7 +1800,7 @@ static PHP_METHOD(swoole_http_client_coro, execute)
 
 static PHP_METHOD(swoole_http_client_coro, get)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
+    http_client* phc = swoole_get_phc(getThis());
     char *uri = NULL;
     size_t uri_len = 0;
 
@@ -1832,7 +1815,7 @@ static PHP_METHOD(swoole_http_client_coro, get)
 
 static PHP_METHOD(swoole_http_client_coro, post)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
+    http_client* phc = swoole_get_phc(getThis());
     char *uri = NULL;
     size_t uri_len = 0;
     zval *post_data;
@@ -1850,9 +1833,9 @@ static PHP_METHOD(swoole_http_client_coro, post)
 
 static PHP_METHOD(swoole_http_client_coro, download)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
-    char *uri = NULL;
-    size_t uri_len = 0;
+    http_client* phc = swoole_get_phc(getThis());
+    char *uri;
+    size_t uri_len;
     zval *download_file;
     zend_long offset = 0;
 
@@ -1871,7 +1854,7 @@ static PHP_METHOD(swoole_http_client_coro, download)
 
 static PHP_METHOD(swoole_http_client_coro, upgrade)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
+    http_client* phc = swoole_get_phc(getThis());
     char *uri = NULL;
     size_t uri_len = 0;
 
@@ -1884,8 +1867,8 @@ static PHP_METHOD(swoole_http_client_coro, upgrade)
 
 static PHP_METHOD(swoole_http_client_coro, push)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
-    zval *zdata = NULL;
+    http_client* phc = swoole_get_phc(getThis());
+    zval *zdata;
     zend_long opcode = WEBSOCKET_OPCODE_TEXT;
     zend_bool fin = 1;
 
@@ -1901,7 +1884,7 @@ static PHP_METHOD(swoole_http_client_coro, push)
 
 static PHP_METHOD(swoole_http_client_coro, recv)
 {
-    PHPHttpClient *phc = swoole_get_phc(getThis());
+    http_client *phc = swoole_get_phc(getThis());
     double timeout = phc->timeout;
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
@@ -1922,8 +1905,7 @@ static PHP_METHOD(swoole_http_client_coro, recv)
 
 static PHP_METHOD(swoole_http_client_coro, close)
 {
-    PHPHttpClient* phc = swoole_get_phc(getThis());
+    http_client* phc = swoole_get_phc(getThis());
 
     RETURN_BOOL(phc->close());
 }
-#endif
