@@ -27,77 +27,74 @@
 
 extern swString *swoole_http_buffer;
 
-namespace swoole
+class http2_stream
 {
-    class http2_stream
+    public:
+    http_context* ctx;
+    // uint8_t priority; // useless now
+    uint32_t stream_id;
+    // flow control
+    uint32_t send_window;
+    uint32_t recv_window;
+
+    http2_stream(int _fd, uint32_t _stream_id)
     {
-        public:
-        http_context* ctx;
-        // uint8_t priority; // useless now
-        uint32_t stream_id;
-        // flow control
-        uint32_t send_window;
-        uint32_t recv_window;
+        ctx = swoole_http_context_new(_fd);
+        ctx->stream = (void *) this;
+        stream_id = _stream_id;
+        send_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+        recv_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+    }
 
-        http2_stream(int _fd, uint32_t _stream_id)
-        {
-            ctx = swoole_http_context_new(_fd);
-            ctx->stream = (void *) this;
-            stream_id = _stream_id;
-            send_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-            recv_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-        }
-
-        ~http2_stream()
-        {
-            swoole_http_context_free(ctx);
-        }
-    };
-
-    class http2_session
+    ~http2_stream()
     {
-        public:
-        int fd;
-        std::unordered_map<int, http2_stream*> streams;
+        swoole_http_context_free(ctx);
+    }
+};
 
-        nghttp2_hd_inflater *inflater;
-        nghttp2_hd_deflater *deflater;
+class http2_session
+{
+    public:
+    int fd;
+    std::unordered_map<int, http2_stream*> streams;
 
-        // flow control
-        uint32_t send_window;
-        uint32_t recv_window;
-        uint32_t max_concurrent_streams;
-        uint32_t max_frame_size;
+    nghttp2_hd_inflater *inflater;
+    nghttp2_hd_deflater *deflater;
 
-        http2_session(int _fd)
+    // flow control
+    uint32_t send_window;
+    uint32_t recv_window;
+    uint32_t max_concurrent_streams;
+    uint32_t max_frame_size;
+
+    http2_session(int _fd)
+    {
+        fd = _fd;
+        send_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+        recv_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+        max_concurrent_streams = SW_HTTP2_MAX_MAX_CONCURRENT_STREAMS;
+        max_frame_size = SW_HTTP2_MAX_MAX_FRAME_SIZE;
+        deflater = nullptr;
+        inflater = nullptr;
+    }
+
+    ~http2_session()
+    {
+        if (inflater)
         {
-            fd = _fd;
-            send_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-            recv_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-            max_concurrent_streams = SW_HTTP2_MAX_MAX_CONCURRENT_STREAMS;
-            max_frame_size = SW_HTTP2_MAX_MAX_FRAME_SIZE;
-            deflater = nullptr;
-            inflater = nullptr;
+            nghttp2_hd_inflate_del(inflater);
+        }
+        if (deflater)
+        {
+            nghttp2_hd_deflate_del(deflater);
         }
 
-        ~http2_session()
+        for(std::unordered_map<int, http2_stream*>::iterator iter = streams.begin(); iter != streams.end(); iter++)
         {
-            if (inflater)
-            {
-                nghttp2_hd_inflate_del(inflater);
-            }
-            if (deflater)
-            {
-                nghttp2_hd_deflate_del(deflater);
-            }
-
-            for(std::unordered_map<int, http2_stream*>::iterator iter = streams.begin(); iter != streams.end(); iter++)
-            {
-                delete iter->second;
-            }
+            delete iter->second;
         }
-    };
-}
+    }
+};
 
 using namespace swoole;
 
@@ -190,7 +187,7 @@ static sw_inline void http2_onRequest(http_context *ctx, int from_fd)
 
     if (SwooleG.enable_coroutine)
     {
-        if (sw_coro_create(fci_cache, 2, args) < 0)
+        if (PHPCoroutine::create(fci_cache, 2, args) < 0)
         {
             swoole_php_error(E_WARNING, "create Http2 onRequest coroutine error.");
             serv->factory.end(&serv->factory, fd);
@@ -262,11 +259,11 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
             {
                 header_flag |= HTTP_HEADER_CONTENT_TYPE;
             }
-            if (ZVAL_IS_NULL(value))
+            if (!ZVAL_IS_NULL(value))
             {
-                continue;
+                convert_to_string(value);
+                http2_add_header(&nv[index++], key, keylen, Z_STRVAL_P(value), Z_STRLEN_P(value));
             }
-            http2_add_header(&nv[index++], key, keylen, Z_STRVAL_P(value), Z_STRLEN_P(value));
         }
         SW_HASHTABLE_FOREACH_END();
         (void)type;
@@ -782,7 +779,6 @@ int swoole_http2_onFrame(swConnection *conn, swEventData *req)
             add_assoc_long(zserver, "remote_port", swConnection_get_port(conn));
             add_assoc_string(zserver, "remote_addr", swConnection_get_ip(conn));
             add_assoc_string(zserver, "server_protocol", (char *) "HTTP/2");
-            add_assoc_string(zserver, "server_software", (char *) SW_HTTP_SERVER_SOFTWARE);
         }
         else
         {
