@@ -22,6 +22,8 @@
 #include "swoole_coroutine.h"
 #include "swoole_mysql.h"
 
+using namespace swoole;
+
 static PHP_METHOD(swoole_mysql_coro, __construct);
 static PHP_METHOD(swoole_mysql_coro, __destruct);
 static PHP_METHOD(swoole_mysql_coro, connect);
@@ -156,7 +158,7 @@ static zend_object *swoole_mysql_coro_create_object(zend_class_entry *ce)
     object->handlers = &swoole_mysql_coro_handlers;
     object_properties_init(object, ce);
 
-    coro_check();
+    PHPCoroutine::check();
 
     mysql_client *client = (mysql_client *) emalloc(sizeof(mysql_client));
     bzero(client, sizeof(mysql_client));
@@ -204,7 +206,7 @@ int mysql_query(zval *zobject, mysql_client *client, swString *sql, zval *callba
 static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *params)
 {
 
-    sw_coro_check_bind("mysql client", client->cid);
+    PHPCoroutine::check_bind("mysql client", client->cid);
 
     if (!client->cli)
     {
@@ -269,7 +271,7 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
     if (param_count != 0)
     {
        //null bitmap
-       char *null_start = p;
+       size_t null_start_offset = p - mysql_request_buffer->str;
        unsigned int map_size = (param_count + 7) / 8;
        memset(p, 0, map_size);
        p += map_size;
@@ -280,7 +282,7 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
        p += 1;
        mysql_request_buffer->length += 1;
 
-       char *type_start = p;
+       size_t type_start_offset = p - mysql_request_buffer->str;
        p += param_count * 2;
        mysql_request_buffer->length += param_count * 2;
 
@@ -289,12 +291,12 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
        {
             if (ZVAL_IS_NULL(value))
             {
-                *(null_start + (index / 8)) |= (1UL << (index % 8));
-                mysql_int2store(type_start + (index * 2), SW_MYSQL_TYPE_NULL);
+                *((mysql_request_buffer->str + null_start_offset) + (index / 8)) |= (1UL << (index % 8));
+                mysql_int2store((mysql_request_buffer->str + type_start_offset) + (index * 2), SW_MYSQL_TYPE_NULL);
             }
             else
             {
-                mysql_int2store(type_start + (index * 2), SW_MYSQL_TYPE_VAR_STRING);
+                mysql_int2store((mysql_request_buffer->str + type_start_offset) + (index * 2), SW_MYSQL_TYPE_VAR_STRING);
                 ZVAL_DUP(&_value, value);
                 value = &_value;
                 convert_to_string(value);
@@ -723,7 +725,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     else
     {
-        connector->timeout = COROG.socket_connect_timeout;
+        connector->timeout = PHPCoroutine::socket_connect_timeout;
     }
     if (php_swoole_array_get_value(_ht, "charset", value))
     {
@@ -833,10 +835,10 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     _socket->object = client;
     _socket->active = 0;
 
-    php_context *context = (php_context *) swoole_get_property(getThis(), 0);
+    php_coro_context *context = (php_coro_context *) swoole_get_property(getThis(), 0);
     if (!context)
     {
-        context = (php_context *) emalloc(sizeof(php_context));
+        context = (php_coro_context *) emalloc(sizeof(php_coro_context));
         swoole_set_property(getThis(), 0, context);
     }
     context->state = SW_CORO_CONTEXT_RUNNING;
@@ -844,11 +846,10 @@ static PHP_METHOD(swoole_mysql_coro, connect)
 
     if (connector->timeout > 0)
     {
-        connector->timer = swTimer_add(&SwooleG.timer, (int) (connector->timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+        connector->timer = swTimer_add(&SwooleG.timer, (long) (connector->timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
     }
-    client->cid = sw_get_current_cid();
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    client->cid = PHPCoroutine::get_cid();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_mysql_coro, query)
@@ -871,9 +872,9 @@ static PHP_METHOD(swoole_mysql_coro, query)
         RETURN_FALSE;
     }
 
-    sw_coro_check_bind("mysql client", client->cid);
+    PHPCoroutine::check_bind("mysql client", client->cid);
 
-    double timeout = COROG.socket_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|d", &sql.str, &sql.length, &timeout) == FAILURE)
     {
@@ -892,10 +893,10 @@ static PHP_METHOD(swoole_mysql_coro, query)
     }
 
     client->state = SW_MYSQL_STATE_READ_START;
-    php_context *context = (php_context *) swoole_get_property(getThis(), 0);
+    php_coro_context *context = (php_coro_context *) swoole_get_property(getThis(), 0);
     if (timeout > 0)
     {
-        client->timer = swTimer_add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+        client->timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
         if (client->timer && client->defer)
         {
             context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
@@ -907,9 +908,8 @@ static PHP_METHOD(swoole_mysql_coro, query)
         RETURN_TRUE;
     }
     client->suspending = 1;
-    client->cid = sw_get_current_cid();
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    client->cid = PHPCoroutine::get_cid();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_mysql_coro, nextResult)
@@ -954,7 +954,7 @@ static void swoole_mysql_coro_query_transcation(const char* command, uint8_t in_
         RETURN_FALSE;
     }
 
-    sw_coro_check_bind("mysql client", client->cid);
+    PHPCoroutine::check_bind("mysql client", client->cid);
 
     // we deny the dangerous operation of transaction
     // if developers need use defer to begin transaction, they can use query("begin/commit/rollback") with defer
@@ -990,20 +990,20 @@ static void swoole_mysql_coro_query_transcation(const char* command, uint8_t in_
     }
     else
     {
-        double timeout = COROG.socket_timeout;
+        double timeout = PHPCoroutine::socket_timeout;
         if (zend_parse_parameters(ZEND_NUM_ARGS(), "|d", &timeout) == FAILURE)
         {
             RETURN_FALSE;
         }
-        php_context *context = (php_context *) swoole_get_property(getThis(), 0);
+        php_coro_context *context = (php_coro_context *) swoole_get_property(getThis(), 0);
         if (timeout > 0)
         {
-            client->timer = swTimer_add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+            client->timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
         }
-        client->cid = sw_get_current_cid();
-        sw_coro_save(return_value, context);
-        coro_use_return_value();
-        sw_coro_yield();
+        client->cid = PHPCoroutine::get_cid();
+        // coro_use_return_value
+        *(zend_uchar *) &execute_data->prev_execute_data->opline->result_type = IS_VAR;
+        PHPCoroutine::yield_m(return_value, context);
         // resume true
         if (Z_BVAL_P(return_value))
         {
@@ -1060,7 +1060,7 @@ static PHP_METHOD(swoole_mysql_coro, recv)
         RETURN_FALSE;
     }
 
-    sw_coro_check_bind("mysql client", client->cid);
+    PHPCoroutine::check_bind("mysql client", client->cid);
 
     if (client->iowait == SW_MYSQL_CORO_STATUS_DONE)
     {
@@ -1079,10 +1079,9 @@ static PHP_METHOD(swoole_mysql_coro, recv)
     }
 
     client->suspending = 1;
-    client->cid = sw_get_current_cid();
-    php_context *context = (php_context *) swoole_get_property(getThis(), 0);
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    client->cid = PHPCoroutine::get_cid();
+    php_coro_context *context = (php_coro_context *) swoole_get_property(getThis(), 0);
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_mysql_coro, prepare)
@@ -1105,9 +1104,9 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
         RETURN_FALSE;
     }
 
-    sw_coro_check_bind("mysql client", client->cid);
+    PHPCoroutine::check_bind("mysql client", client->cid);
 
-    double timeout = COROG.socket_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|d", &sql.str, &sql.length, &timeout) == FAILURE)
     {
@@ -1151,15 +1150,14 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
         RETURN_TRUE;
     }
 
-    php_context *context = (php_context *) swoole_get_property(getThis(), 0);
+    php_coro_context *context = (php_coro_context *) swoole_get_property(getThis(), 0);
     if (timeout > 0)
     {
-        client->timer = swTimer_add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+        client->timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
     }
     client->suspending = 1;
-    client->cid = sw_get_current_cid();
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    client->cid = PHPCoroutine::get_cid();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_mysql_coro_statement, execute)
@@ -1179,7 +1177,7 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
         RETURN_FALSE;
     }
 
-    double timeout = COROG.socket_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|ad", &params, &timeout) == FAILURE)
     {
@@ -1196,10 +1194,10 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
         RETURN_FALSE;
     }
 
-    php_context *context = (php_context *) swoole_get_property(client->object, 0);
+    php_coro_context *context = (php_coro_context *) swoole_get_property(client->object, 0);
     if (timeout > 0)
     {
-        client->timer = swTimer_add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+        client->timer = swTimer_add(&SwooleG.timer, (long) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
         if (client->timer && client->defer)
         {
             context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
@@ -1211,9 +1209,8 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
         RETURN_TRUE;
     }
     client->suspending = 1;
-    client->cid = sw_get_current_cid();
-    sw_coro_save(return_value, context);
-    sw_coro_yield();
+    client->cid = PHPCoroutine::get_cid();
+    PHPCoroutine::yield_m(return_value, context);
 }
 
 static PHP_METHOD(swoole_mysql_coro_statement, fetch)
@@ -1353,7 +1350,7 @@ static PHP_METHOD(swoole_mysql_coro, escape)
 {
     swString str;
     bzero(&str, sizeof(str));
-    long flags;
+    zend_long flags = 0;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|l", &str.str, &str.length, &flags) == FAILURE)
     {
@@ -1435,7 +1432,7 @@ static void swoole_mysql_coro_free_object(zend_object *object)
         swoole_set_object_by_handle(handle, NULL);
     }
 
-    php_context *context = (php_context *) swoole_get_property_by_handle(handle, 0);
+    php_coro_context *context = (php_coro_context *) swoole_get_property_by_handle(handle, 0);
     if (context)
     {
         efree(context);
@@ -1463,11 +1460,11 @@ static int swoole_mysql_coro_onError(swReactor *reactor, swEvent *event)
     }
     client->suspending = 0;
     client->cid = 0;
-    php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-    int ret = sw_coro_resume(sw_current_context, result, retval);
+    php_coro_context *sw_current_context = (php_coro_context *) swoole_get_property(zobject, 0);
+    int ret = PHPCoroutine::resume_m(sw_current_context, result, retval);
     sw_zval_free(result);
 
-    if (ret == CORO_END && retval)
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -1512,10 +1509,10 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
 
     client->cid = 0;
 
-    php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-    int ret = sw_coro_resume(sw_current_context, result, retval);
+    php_coro_context *sw_current_context = (php_coro_context *) swoole_get_property(zobject, 0);
+    int ret = PHPCoroutine::resume_m(sw_current_context, result, retval);
     zval_ptr_dtor(result);
-    if (ret == CORO_END && retval)
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -1525,7 +1522,7 @@ static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
 {
     zval *result = sw_malloc_zval();;
     zval *retval = NULL;
-    php_context *ctx = (php_context *) tnode->data;
+    php_coro_context *ctx = (php_coro_context *) tnode->data;
     zval _zobject = ctx->coro_params;
     zval *zobject = & _zobject;
 
@@ -1533,17 +1530,17 @@ static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
 
     mysql_client *client = (mysql_client *) swoole_get_object(zobject);
 
-    if (client->iowait == SW_MYSQL_CORO_STATUS_CLOSED)
+    if (client->handshake != SW_MYSQL_HANDSHAKE_COMPLETED)
     {
-        zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "connect timeout");
+        zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_error"), "connect timeout");
+        zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_errno"), ETIMEDOUT);
     }
     else
     {
         zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "query timeout");
+        zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"), ETIMEDOUT);
     }
 
-    zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"), ETIMEDOUT);
-
     //timeout close conncttion
     client->timer = NULL;
     client->state = SW_MYSQL_STATE_QUERY;
@@ -1557,9 +1554,9 @@ static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
     client->suspending = 0;
     client->cid = 0;
 
-    int ret = sw_coro_resume(ctx, result, retval);
+    int ret = PHPCoroutine::resume_m(ctx, result, retval);
 
-    if (ret == CORO_END && retval)
+    if (ret == SW_CORO_ERR_END && retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -1868,10 +1865,10 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             client->suspending = 0;
             client->cid = 0;
 
-            php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-            ret = sw_coro_resume(sw_current_context, result, retval);
+            php_coro_context *sw_current_context = (php_coro_context *) swoole_get_property(zobject, 0);
+            ret = PHPCoroutine::resume_m(sw_current_context, result, retval);
             sw_zval_free(result);
-            if (ret == CORO_END && retval)
+            if (ret == SW_CORO_ERR_END && retval)
             {
                 zval_ptr_dtor(retval);
             }
@@ -1940,13 +1937,13 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             client->iowait = SW_MYSQL_CORO_STATUS_READY;
             client->cid = 0;
 
-            php_context *sw_current_context = (php_context *) swoole_get_property(zobject, 0);
-            ret = sw_coro_resume(sw_current_context, result, retval);
+            php_coro_context *sw_current_context = (php_coro_context *) swoole_get_property(zobject, 0);
+            ret = PHPCoroutine::resume_m(sw_current_context, result, retval);
             if (result)
             {
                 sw_zval_free(result);
             }
-            if (ret == CORO_END && retval)
+            if (ret == SW_CORO_ERR_END && retval)
             {
                 zval_ptr_dtor(retval);
             }
