@@ -71,15 +71,17 @@ typedef struct
 } process_pool_property;
 
 static zend_class_entry swoole_process_pool_ce;
-static zend_class_entry *swoole_process_pool_class_entry_ptr;
+static zend_class_entry *swoole_process_pool_ce_ptr;
+static zend_object_handlers swoole_process_pool_handlers;
 static swProcessPool *current_pool;
 static zval *current_process = NULL;
 
 void swoole_process_pool_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_process_pool_ce, "swoole_process_pool", "Swoole\\Process\\Pool", swoole_process_pool_methods);
-    swoole_process_pool_class_entry_ptr = zend_register_internal_class(&swoole_process_pool_ce);
-    SWOOLE_CLASS_ALIAS(swoole_process_pool, "Swoole\\Process\\Pool");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_process_pool, "Swoole\\Process\\Pool", "swoole_process_pool", NULL, swoole_process_pool_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_process_pool, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_process_pool, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_process_pool, zend_class_unset_property_deny);
 }
 
 static void php_swoole_process_pool_onWorkerStart(swProcessPool *pool, int worker_id)
@@ -109,7 +111,7 @@ static void php_swoole_process_pool_onWorkerStart(swProcessPool *pool, int worke
     {
         swoole_php_fatal_error(E_WARNING, "onWorkerStart handler error.");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -129,7 +131,7 @@ static void php_swoole_process_pool_onMessage(swProcessPool *pool, char *data, u
 
     zval *zobject = (zval *) pool->ptr;
     zval *zdata;
-    zval *retval;
+    zval *retval = NULL;
 
     SW_MAKE_STD_ZVAL(zdata);
     ZVAL_STRINGL(zdata, data, length);
@@ -144,7 +146,7 @@ static void php_swoole_process_pool_onMessage(swProcessPool *pool, char *data, u
     {
         swoole_php_fatal_error(E_WARNING, "onMessage handler error.");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -178,7 +180,7 @@ static void php_swoole_process_pool_onWorkerStop(swProcessPool *pool, int worker
     {
         swoole_php_fatal_error(E_WARNING, "onWorkerStop handler error.");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -207,9 +209,9 @@ static void php_swoole_process_pool_signal_handler(int sig)
 
 static PHP_METHOD(swoole_process_pool, __construct)
 {
-    long worker_num;
-    long ipc_type = SW_IPC_NONE;
-    long msgq_key = 0;
+    zend_long worker_num;
+    zend_long ipc_type = SW_IPC_NONE;
+    zend_long msgq_key = 0;
 
     //only cli env
     if (!SWOOLE_G(cli))
@@ -231,14 +233,14 @@ static PHP_METHOD(swoole_process_pool, __construct)
 
     if (worker_num <= 0)
     {
-        zend_throw_exception_ex(swoole_exception_class_entry_ptr, errno, "invalid worker_num");
+        zend_throw_exception_ex(swoole_exception_ce_ptr, errno, "invalid worker_num");
         RETURN_FALSE;
     }
 
     swProcessPool *pool = emalloc(sizeof(swProcessPool));
     if (swProcessPool_create(pool, worker_num, 0, (key_t) msgq_key, ipc_type) < 0)
     {
-        zend_throw_exception_ex(swoole_exception_class_entry_ptr, errno, "failed to create process pool");
+        zend_throw_exception_ex(swoole_exception_ce_ptr, errno, "failed to create process pool");
         RETURN_FALSE;
     }
 
@@ -246,7 +248,7 @@ static PHP_METHOD(swoole_process_pool, __construct)
     {
         if (swProcessPool_set_protocol(pool, 0, SW_BUFFER_INPUT_SIZE) < 0)
         {
-            zend_throw_exception_ex(swoole_exception_class_entry_ptr, errno, "failed to create process pool");
+            zend_throw_exception_ex(swoole_exception_ce_ptr, errno, "failed to create process pool");
             RETURN_FALSE;
         }
     }
@@ -334,8 +336,8 @@ static PHP_METHOD(swoole_process_pool, listen)
 {
     char *host;
     size_t l_host;
-    long port;
-    long backlog = 2048;
+    zend_long port = 0;
+    zend_long backlog = 2048;
 
     swProcessPool *pool = swoole_get_object(getThis());
 
@@ -454,9 +456,9 @@ static PHP_METHOD(swoole_process_pool, getProcess)
     if (current_process == NULL)
     {
         swWorker *worker = &current_pool->workers[SwooleWG.id];
-        object_init_ex(&object, swoole_process_class_entry_ptr);
-        zend_update_property_long(swoole_process_class_entry_ptr, &object, ZEND_STRL("id"), SwooleWG.id);
-        zend_update_property_long(swoole_process_class_entry_ptr, &object, ZEND_STRL("pid"), getpid());
+        object_init_ex(&object, swoole_process_ce_ptr);
+        zend_update_property_long(swoole_process_ce_ptr, &object, ZEND_STRL("id"), SwooleWG.id);
+        zend_update_property_long(swoole_process_ce_ptr, &object, ZEND_STRL("pid"), getpid());
         swoole_set_object(&object, worker);
         current_process = &object;
     }
