@@ -24,6 +24,8 @@ BEGIN_EXTERN_C()
 #include "ext/standard/php_string.h"
 END_EXTERN_C()
 
+using namespace swoole;
+
 static zend_class_entry swoole_redis_server_ce;
 static zend_class_entry *swoole_redis_server_ce_ptr;
 static zend_object_handlers swoole_redis_server_handlers;
@@ -201,7 +203,7 @@ static int redis_onReceive(swServer *serv, swEventData *req)
 
     if (SwooleG.enable_coroutine)
     {
-        if (sw_coro_create(fci_cache, 2, args) < 0)
+        if (PHPCoroutine::create(fci_cache, 2, args) < 0)
         {
             swoole_php_error(E_WARNING, "create redis server onReceive coroutine error.");
         }
@@ -288,7 +290,7 @@ static PHP_METHOD(swoole_redis_server, setHandler)
         RETURN_FALSE;
     }
 
-    if (command_len <= 0 || command_len >= SW_REDIS_MAX_COMMAND_SIZE)
+    if (command_len == 0 || command_len >= SW_REDIS_MAX_COMMAND_SIZE)
     {
         swoole_php_fatal_error(E_ERROR, "invalid command.");
         RETURN_FALSE;
@@ -331,7 +333,7 @@ static PHP_METHOD(swoole_redis_server, setHandler)
 
 static PHP_METHOD(swoole_redis_server, format)
 {
-    long type;
+    zend_long type;
     zval *value = NULL;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|z", &type, &value) == FAILURE)
@@ -459,7 +461,7 @@ static PHP_METHOD(swoole_redis_server, format)
         int keytype;
 
         SW_HASHTABLE_FOREACH_START2(Z_ARRVAL_P(value), key, keylen, keytype, item)
-            if (key == NULL || keylen <= 0)
+            if (key == NULL || keylen == 0)
             {
                 continue;
             }
