@@ -152,7 +152,6 @@ static auto block_io_functions = {
     "file_get_contents",
     "curl_init",
     "stream_select",
-    "stream_create",
     "pcntl_fork",
     "popen",
     "socket_select",
@@ -382,7 +381,7 @@ static inline int socket_connect(php_stream *stream, Socket *sock, php_stream_xp
         ip_address = parse_ip_address_ex(xparam->inputs.name, xparam->inputs.namelen, &portno, xparam->want_errortext,
                 &xparam->outputs.error_text);
         host = ip_address;
-        if (sock->_sock_type == SOCK_STREAM)
+        if (sock->sock_type == SOCK_STREAM)
         {
             int sockoptval = 1;
             setsockopt(sock->get_fd(), IPPROTO_TCP, TCP_NODELAY, (char*) &sockoptval, sizeof(sockoptval));
@@ -637,7 +636,7 @@ static inline int socket_xport_api(php_stream *stream, Socket *sock, php_stream_
         break;
     case STREAM_XPORT_OP_BIND:
     {
-        if (sock->_sock_domain != AF_UNIX)
+        if (sock->sock_domain != AF_UNIX)
         {
             zval *tmpzval = NULL;
             int sockoptval = 1;
@@ -861,7 +860,7 @@ static php_stream *socket_create(
     php_stream *stream = NULL;
     Socket *sock;
 
-    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    if (unlikely(SwooleG.main_reactor == nullptr || !Coroutine::get_current()))
     {
         return php_socket_create(proto, protolen, resourcename, resourcenamelen, persistent_id, options, flags, timeout, context STREAMS_CC);
     }
@@ -895,8 +894,8 @@ static php_stream *socket_create(
     if (unlikely(sock->socket == nullptr))
     {
         _failed:
-        delete sock;
         swoole_php_fatal_error(E_WARNING, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
+        delete sock;
         return NULL;
     }
 
@@ -921,7 +920,7 @@ static php_stream *socket_create(
     return stream;
 }
 
-bool sw_enable_coroutine_hook(int flags)
+bool PHPCoroutine::enable_hook(int flags)
 {
     if (unlikely(enable_strict_mode))
     {
@@ -1011,7 +1010,7 @@ bool sw_enable_coroutine_hook(int flags)
     return true;
 }
 
-bool sw_disable_coroutine_hook()
+bool PHPCoroutine::disable_hook()
 {
     if (!hook_init)
     {
@@ -1092,7 +1091,7 @@ static PHP_METHOD(swoole_runtime, enableCoroutine)
         {
             RETURN_FALSE;
         }
-        sw_enable_coroutine_hook(flags);
+        PHPCoroutine::enable_hook(flags);
     }
     else
     {
@@ -1100,7 +1099,7 @@ static PHP_METHOD(swoole_runtime, enableCoroutine)
         {
             RETURN_FALSE;
         }
-        sw_disable_coroutine_hook();
+        PHPCoroutine::disable_hook();
     }
 }
 
@@ -1117,7 +1116,7 @@ static PHP_FUNCTION(_sleep)
         RETURN_FALSE;
     }
 
-    if (num >= 0.001 && sw_coro_is_in())
+    if (num >= 0.001 && PHPCoroutine::is_in())
     {
         php_swoole_check_reactor();
         RETURN_LONG(Coroutine::sleep((double ) num) < 0 ? num : 0);
@@ -1142,7 +1141,7 @@ static PHP_FUNCTION(_usleep)
     }
     double _time = (double) num / 1000000;
 
-    if (_time >= 0.001 && sw_coro_is_in())
+    if (_time >= 0.001 && PHPCoroutine::is_in())
     {
         php_swoole_check_reactor();
         Coroutine::sleep((double) num / 1000000);
@@ -1172,7 +1171,7 @@ static PHP_FUNCTION(_time_nanosleep)
         RETURN_FALSE;
     }
     double _time = (double) tv_sec + (double) tv_nsec / 1000000000.00;
-    if (_time >= 0.001 && sw_coro_is_in())
+    if (_time >= 0.001 && PHPCoroutine::is_in())
     {
         php_swoole_check_reactor();
         Coroutine::sleep(_time);
@@ -1231,7 +1230,7 @@ static PHP_FUNCTION(_time_sleep_until)
     php_req.tv_nsec = (long) ((c_ts - php_req.tv_sec) * 1000000000.00);
 
     double _time = (double) php_req.tv_sec + (double) php_req.tv_nsec / 1000000000.00;
-    if (_time >= 0.001 && sw_coro_is_in())
+    if (_time >= 0.001 && PHPCoroutine::is_in())
     {
         php_swoole_check_reactor();
         Coroutine::sleep(_time);
