@@ -59,7 +59,7 @@ static PHP_METHOD(swoole_socket_coro, close);
 static PHP_METHOD(swoole_socket_coro, getSocket);
 #endif
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_socket_coro_construct, 0, 0, 3)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_socket_coro_construct, 0, 0, 2)
     ZEND_ARG_INFO(0, domain)
     ZEND_ARG_INFO(0, type)
     ZEND_ARG_INFO(0, protocol)
@@ -183,29 +183,31 @@ void swoole_socket_coro_init(int module_number)
 
 static PHP_METHOD(swoole_socket_coro, __construct)
 {
-    zend_long domain, type, protocol;
+    zend_long domain, type, protocol = 0;
 
-    ZEND_PARSE_PARAMETERS_START(3, 3)
+    ZEND_PARSE_PARAMETERS_START(2, 3)
         Z_PARAM_LONG(domain)
         Z_PARAM_LONG(type)
+        Z_PARAM_OPTIONAL
         Z_PARAM_LONG(protocol)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
     php_swoole_check_reactor();
 
     socket_coro *sock = (socket_coro *) swoole_socket_coro_fetch_object(Z_OBJ_P(getThis()));
-    sock->socket = new Socket(get_socket_type(domain, type, protocol));
+    sock->socket = new Socket((int)domain, (int)type, (int)protocol);
     if (unlikely(sock->socket->socket == nullptr))
     {
-        delete sock->socket;
         zend_throw_exception_ex(
             swoole_socket_coro_exception_ce_ptr, errno,
             "new Socket() failed. Error: %s [%d]", strerror(errno), errno
         );
+        delete sock->socket;
+        sock->socket = nullptr;
         RETURN_FALSE;
     }
     sock->domain = domain;
-    sock->socket->set_timeout(COROG.socket_timeout);
+    sock->socket->set_timeout(PHPCoroutine::socket_timeout);
 }
 
 static PHP_METHOD(swoole_socket_coro, bind)
@@ -249,7 +251,7 @@ static PHP_METHOD(swoole_socket_coro, listen)
 
 static PHP_METHOD(swoole_socket_coro, accept)
 {
-    double timeout = COROG.socket_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
         Z_PARAM_OPTIONAL
@@ -258,8 +260,10 @@ static PHP_METHOD(swoole_socket_coro, accept)
 
     socket_coro *sock = swoole_get_socket_coro(getThis());
 
-    sock->socket->set_timeout(timeout, true);
+    double persistent_timeout = sock->socket->get_timeout();
+    sock->socket->set_timeout(timeout);
     Socket *conn = sock->socket->accept();
+    sock->socket->set_timeout(persistent_timeout);
     if (conn)
     {
         zend_object *client = swoole_socket_coro_create_object(swoole_socket_coro_ce_ptr);
@@ -276,10 +280,8 @@ static PHP_METHOD(swoole_socket_coro, accept)
 
 static PHP_METHOD(swoole_socket_coro, recv)
 {
-    coro_check();
-
     zend_long length = SW_BUFFER_SIZE_BIG;
-    double timeout = COROG.socket_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(0, 2)
         Z_PARAM_OPTIONAL
@@ -294,9 +296,11 @@ static PHP_METHOD(swoole_socket_coro, recv)
 
     socket_coro *sock = swoole_get_socket_coro(getThis());
 
-    sock->socket->set_timeout(timeout, true);
     zend_string *buf = zend_string_alloc(length, 0);
+    double persistent_timeout = sock->socket->get_timeout();
+    sock->socket->set_timeout(timeout);
     ssize_t bytes = sock->socket->recv(ZSTR_VAL(buf), length);
+    sock->socket->set_timeout(persistent_timeout);
     if (bytes < 0)
     {
         zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
@@ -319,7 +323,7 @@ static PHP_METHOD(swoole_socket_coro, recv)
 static PHP_METHOD(swoole_socket_coro, recvfrom)
 {
     zval *peername;
-    double timeout = COROG.socket_timeout;
+    double timeout = PHPCoroutine::socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(1, 2)
         Z_PARAM_ZVAL_EX(peername, 0, 1)
@@ -330,8 +334,10 @@ static PHP_METHOD(swoole_socket_coro, recvfrom)
     socket_coro *sock = swoole_get_socket_coro(getThis());
 
     zend_string *buf = zend_string_alloc(SW_BUFFER_SIZE_BIG, 0);
-    sock->socket->set_timeout(timeout, true);
+    double persistent_timeout = sock->socket->get_timeout();
+    sock->socket->set_timeout(timeout);
     ssize_t bytes = sock->socket->recvfrom(ZSTR_VAL(buf), SW_BUFFER_SIZE_BIG);
+    sock->socket->set_timeout(persistent_timeout);
     if (bytes < 0)
     {
         zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
@@ -370,9 +376,9 @@ static PHP_METHOD(swoole_socket_coro, recvfrom)
 
 static PHP_METHOD(swoole_socket_coro, send)
 {
-    double timeout = COROG.socket_timeout;
     char *data;
     size_t l_data;
+    double timeout = PHPCoroutine::socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(1, 2)
         Z_PARAM_STRING(data, l_data)
@@ -382,8 +388,10 @@ static PHP_METHOD(swoole_socket_coro, send)
 
     socket_coro *sock = swoole_get_socket_coro(getThis());
 
-    sock->socket->set_timeout(timeout, true);
+    double persistent_timeout = sock->socket->get_timeout();
+    sock->socket->set_timeout(timeout);
     ssize_t retval = sock->socket->send(data, l_data);
+    sock->socket->set_timeout(persistent_timeout);
     if (retval < 0)
     {
         zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
@@ -424,8 +432,6 @@ static PHP_METHOD(swoole_socket_coro, sendto)
 
 static PHP_METHOD(swoole_socket_coro, close)
 {
-    coro_check();
-
     socket_coro *sock = swoole_get_socket_coro(getThis());
 
     if (!sock->socket->close())
@@ -514,13 +520,11 @@ static PHP_METHOD(swoole_socket_coro, getpeername)
 
 static PHP_METHOD(swoole_socket_coro, connect)
 {
-    coro_check();
-
     socket_coro *sock = swoole_get_socket_coro(getThis());
     char *host;
     size_t l_host;
     zend_long port = 0;
-    double timeout = COROG.socket_connect_timeout;
+    double timeout = PHPCoroutine::socket_connect_timeout;
 
     ZEND_PARSE_PARAMETERS_START(1, 3)
         Z_PARAM_STRING(host, l_host)
@@ -543,15 +547,10 @@ static PHP_METHOD(swoole_socket_coro, connect)
         }
     }
 
-    sock->socket->set_timeout(timeout, true);
-    if (sock->socket->connect(std::string(host, l_host), port))
-    {
-        RETURN_TRUE;
-    }
-    else
-    {
-        RETURN_FALSE;
-    }
+    sock->socket->set_timeout(timeout);
+    bool ret = sock->socket->connect(std::string(host, l_host), port);
+    sock->socket->set_timeout(PHPCoroutine::socket_timeout);
+    RETURN_BOOL(ret);
 }
 
 #ifdef SWOOLE_SOCKETS_SUPPORT
