@@ -21,6 +21,8 @@
 #include "swoole_coroutine.h"
 #endif
 
+using namespace swoole;
+
 enum swoole_timer_type
 {
     SW_TIMER_TICK, SW_TIMER_AFTER,
@@ -72,7 +74,8 @@ long php_swoole_add_timer(long ms, zval *callback, zval *param, int persistent)
         return SW_ERR;
     }
 
-    if (!(SwooleG.serv && swIsTaskWorker() && SwooleG.serv->task_async == 0))
+    // no server || task process with async mode || user worker
+    if (!SwooleG.serv || (SwooleG.serv && ((swIsTaskWorker() && SwooleG.serv->task_enable_coroutine) || swIsUserWorker())))
     {
         php_swoole_check_reactor();
     }
@@ -163,7 +166,7 @@ void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode)
 
     if (SwooleG.enable_coroutine)
     {
-        if (sw_coro_create(cb->fci_cache, argc, args) < 0)
+        if (PHPCoroutine::create(cb->fci_cache, argc, args) < 0)
         {
             swoole_php_fatal_error(E_WARNING, "create onTimer coroutine error.");
         }
@@ -179,6 +182,11 @@ void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode)
     }
 
     php_swoole_del_timer(tnode);
+
+    if (UNEXPECTED(EG(exception)))
+    {
+        zend_exception_error(EG(exception), E_ERROR);
+    }
 }
 
 void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode)
@@ -201,7 +209,7 @@ void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode)
 
     if (SwooleG.enable_coroutine)
     {
-        if (sw_coro_create(cb->fci_cache, argc, args) < 0)
+        if (PHPCoroutine::create(cb->fci_cache, argc, args) < 0)
         {
             swoole_php_fatal_error(E_WARNING, "create onInterval coroutine error.");
             return;
@@ -221,11 +229,16 @@ void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode)
     {
         php_swoole_del_timer(tnode);
     }
+
+    if (UNEXPECTED(EG(exception)))
+    {
+        zend_exception_error(EG(exception), E_ERROR);
+    }
 }
 
 PHP_FUNCTION(swoole_timer_tick)
 {
-    long after_ms;
+    zend_long after_ms;
     zval *callback;
     zval *param = NULL;
 
